# -*- encoding: utf-8 -*-
from abjad import *
import pytest


def test_scoretools_Container___setitem___slice_01():
    r'''Containers set single leaves correctly in an unspanned structure.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    staff[2:2] = [Note(7, (1, 8))]

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            c'8
            d'8
            g'8
            e'8
            f'8
        }
        '''
        )

    assert inspect(staff).is_well_formed()


def test_scoretools_Container___setitem___slice_02():
    r'''Set single leaf between spanned components.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, staff[:])
    note = Note(7, (1, 8))
    staff[2:2] = [note]

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            c'8 [
            d'8
            g'8
            e'8
            f'8 ]
        }
        '''
        )

    assert inspect(staff).is_well_formed()


def test_scoretools_Container___setitem___slice_03():
    r'''Containers set sequence of leaves between spanned components.
    '''

    notes = [
        Note("c'8"), Note("d'8"), Note("e'8"), 
        Note("f'8"), Note("g'8"), Note("a'8"),
        ]

    beginning = notes[:2]
    middle = notes[2:4]
    end = notes[4:]

    staff = Staff(beginning + end)
    beam = Beam()
    attach(beam, staff[:])

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            c'8 [
            d'8
            g'8
            a'8 ]
        }
        '''
        )

    staff[2:2] = middle

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            c'8 [
            d'8
            e'8
            f'8
            g'8
            a'8 ]
        }
        '''
        )

    assert inspect(staff).is_well_formed()


def test_scoretools_Container___setitem___slice_04():
    r'''Replace sequence of spanned components with a single leaf.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, staff[:])
    note = Note(12, (1, 8))
    staff[1:3] = [note]

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            c'8 [
            c''8
            f'8 ]
        }
        '''
        )

    assert inspect(staff).is_well_formed()


def test_scoretools_Container___setitem___slice_05():
    r'''Replace a sequence of multiple components with
    a different sequence of multiple components.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, staff[:])
    notes = [Note(11, (1, 8)), Note(9, (1, 8)), Note(7, (1, 8))]
    staff[1:3] = notes
        
    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            c'8 [
            b'8
            a'8
            g'8
            f'8 ]
        }
        '''
        )

    assert inspect(staff).is_well_formed()


def test_scoretools_Container___setitem___slice_06():
    r'''Donor and recipient container are the same.
    '''

    staff = Staff("{ c'8 d'8 } { e'8 f'8 }")
    beam = Beam()
    attach(beam, staff.select_leaves())

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                c'8 [
                d'8
            }
            {
                e'8
                f'8 ]
            }
        }
        '''
        )

    sequential = staff[0]
    staff[0:1] = sequential.select_leaves()

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            c'8 [
            d'8
            {
                e'8
                f'8 ]
            }
        }
        '''
        )

    assert inspect(staff).is_well_formed()
    assert len(sequential) == 0


def test_scoretools_Container___setitem___slice_07():
    r'''Donor and recipient container are the same.
    '''

    staff = Staff("{ c'8 d'8 } { e'8 f'8 }")
    beam = Beam()
    attach(beam, staff.select_leaves())

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                c'8 [
                d'8
            }
            {
                e'8
                f'8 ]
            }
        }
        '''
        )

    staff[0:0] = staff[0][:1]

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            c'8
            {
                d'8 [
            }
            {
                e'8
                f'8 ]
            }
        }
        '''
        )

    assert inspect(staff).is_well_formed()


def test_scoretools_Container___setitem___slice_08():
    r'''Donor and recipient container are the same.
    '''

    staff = Staff("{ c'8 d'8 } { e'8 f'8 }")
    beam = Beam()
    attach(beam, staff.select_leaves())

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                c'8 [
                d'8
            }
            {
                e'8
                f'8 ]
            }
        }
        '''
        )

    staff[0:0] = staff[0][:]

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            c'8
            d'8
            {
            }
            {
                e'8 [
                f'8 ]
            }
        }
        '''
        )


def test_scoretools_Container___setitem___slice_09():
    r'''Donor and recipient container are the same.
    '''

    staff = Staff("{ c'8 d'8 } { e'8 f'8 }")
    beam = Beam()
    attach(beam, staff.select_leaves())

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                c'8 [
                d'8
            }
            {
                e'8
                f'8 ]
            }
        }
        '''
        )

    sequential = staff[0]
    staff[0:0] = sequential[:]
    sequential[0:0] = staff[-1][:1]

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            c'8
            d'8
            {
                e'8
            }
            {
                f'8 [ ]
            }
        }
        '''
        )

    assert inspect(staff).is_well_formed()


def test_scoretools_Container___setitem___slice_10():
    r'''Donor and recipient container are the same.
    '''

    staff = Staff("{ c'8 d'8 } { e'8 f'8 }")
    beam = Beam()
    attach(beam, staff.select_leaves())

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                c'8 [
                d'8
            }
            {
                e'8
                f'8 ]
            }
        }
        '''
        )

    staff[0:0] = staff[0][:1]
    staff[len(staff):len(staff)] = staff[-1][-1:]

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            c'8
            {
                d'8 [
            }
            {
                e'8 ]
            }
            f'8
        }
        '''
        )

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            c'8
            {
                d'8 [
            }
            {
                e'8 ]
            }
            f'8
        }
        '''
        )

    assert inspect(staff).is_well_formed()


def test_scoretools_Container___setitem___slice_11():
    r'''Extremely small coequal indices act as zero.
    '''

    voice = Voice("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, voice[:])
    voice[-1000:-1000] = [Rest((1, 8))]

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            r8
            c'8 [
            d'8
            e'8
            f'8 ]
        }
        '''
        )

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            r8
            c'8 [
            d'8
            e'8
            f'8 ]
        }
        '''
        )

    assert inspect(voice).is_well_formed()


def test_scoretools_Container___setitem___slice_12():
    r'''Extremely large, coequal indices work correctly.
    '''

    voice = Voice("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, voice[:])
    voice[1000:1000] = [Rest((1, 8))]

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            c'8 [
            d'8
            e'8
            f'8 ]
            r8
        }
        '''
        )

    assert inspect(voice).is_well_formed()


def test_scoretools_Container___setitem___slice_13():
    r'''You can use setitem to empty the contents of a container.
    When you do this, emptied components withdraw
    from absolutely all spanners.

    On the other hand, if you want to empty a container and
    allow the emptied components to remain embedded within spanners,
    use del(container) instead.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    inner = Container(staff[1:3])
    outer = Container([inner])
    beam = Beam()
    attach(beam, inner[:])

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            c'8
            {
                {
                    d'8 [
                    e'8 ]
                }
            }
            f'8
        }
        '''
        )

    # empty outer container
    outer[:] = []

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            c'8
            {
            }
            f'8
        }
        '''
        )

    # inner container leaves DO withdraw from all spanners
    assert systemtools.TestManager.compare(
        inner,
        r'''
        {
            d'8
            e'8
        }
        '''
        )

    # ALTERNATIVE: use del(container)

    staff = Staff("c'8 d'8 e'8 f'8")
    inner = Container(staff[1:3])
    outer = Container([inner])
    beam = Beam()
    attach(beam, inner[:])

    del(outer[:])

    r'''
    \new Staff {
        c'8
        {
        }
        f'8
    }
    '''

    r'''
    {
        d'8 [
        e'8 ]
    }
    '''

    # inner container leaves DO NOT withdraw from spanners
    assert systemtools.TestManager.compare(
        inner,
        r'''
        {
            d'8 [
            e'8 ]
        }
        '''
        )



# -*- encoding: utf-8 -*-
from abjad.tools.abctools.AbjadObject import AbjadObject


class KeySignature(AbjadObject):
    r'''A key signature.

    ::

        >>> staff = Staff("e'8 fs'8 gs'8 a'8")
        >>> key_signature = KeySignature('e', 'major')
        >>> attach(key_signature, staff)
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> print format(staff)
        \new Staff {
            \key e \major
            e'8
            fs'8
            gs'8
            a'8
        }

    '''

    ### CLASS VARIABLES ###

    _default_positional_input_arguments = (
        repr('c'),
        repr('major'),
        )

    _format_slot = 'opening'

    ### INITIALIZER ###

    def __init__(self, tonic, mode):
        from abjad.tools import pitchtools
        from abjad.tools import scoretools
        from abjad.tools import tonalanalysistools
        tonic = pitchtools.NamedPitchClass(tonic)
        mode = tonalanalysistools.Mode(mode)
        self._tonic = tonic
        self._mode = mode
        self._default_scope = scoretools.Staff

    ### SPECIAL METHODS ###

    def __copy__(self, *args):
        r'''Copies key signature.

        Returns new key signature.
        '''
        return type(self)(
            self._tonic, 
            self._mode,
            )

    def __eq__(self, arg):
        r'''True when `arg` is a key signature with tonic and mode equal
        to key signature. Otherwise false.

        Returns boolean.
        '''
        if isinstance(arg, type(self)):
            if self.tonic == arg.tonic:
                if self.mode == arg.mode:
                    return True
        return False

    def __str__(self):
        r'''String representation of key signature.

        Returns string.
        '''
        return '{!s}-{!s}'.format(self.tonic, self.mode)

    ### PRIVATE PROPERTIES ###

    @property
    def _contents_repr_string(self):
        return '{!r}, {!r}'.format(self.tonic, self.mode)

    @property
    def _lilypond_format(self):
        return r'\key {!s} \{!s}'.format(self.tonic, self.mode)

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        return systemtools.StorageFormatSpecification(
            self,
            is_indented=False,
            positional_argument_values=(
                str(self.tonic),
                str(self.mode),
                ),
            )

    ### PUBLIC PROPERTIES ###

    @property
    def mode(self):
        r'''Mode of signature.

        ::

            >>> key_signature.mode
            Mode('major')

        Returns mode.
        '''
        return self._mode

    @property
    def name(self):
        r'''Name of key signature.

        ::

            >>> key_signature = KeySignature('e', 'major')
            >>> key_signature.name
            'E major'

        Returns string.
        '''
        if self.mode.mode_name == 'major':
            tonic = str(self.tonic).upper()
        else:
            tonic = str(self.tonic)
        return '{!s} {!s}'.format(tonic, self.mode.mode_name)

    @property
    def tonic(self):
        r'''Tonic of key signature.

        ::

            >>> key_signature.tonic
            NamedPitchClass('e')

        Returns named pitch-class.
        '''
        return self._tonic



# -*- encoding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools import scoretools
from abjad.tools import scoretools
from abjad.tools.quantizationtools.AttackPointOptimizer \
	import AttackPointOptimizer


class MeasurewiseAttackPointOptimizer(AttackPointOptimizer):
    r'''Concrete ``AttackPointOptimizer`` instance which attempts to optimize
    attack points in an expression with regard to the effective time 
    signature of that expression.

    Only acts on Measure instances.
    '''

    ### SPECIAL METHODS ###

    def __call__(self, expr):
        r'''Calls measurewise attack-point optimizer.

        Returns none.
        '''
        from abjad.tools import quantizationtools
        assert isinstance(expr, scoretools.Measure)
        meter = metertools.Meter(expr)
        metertools.rewrite_meter(
            expr[:], meter)



# -*- encoding: utf-8 -*-
'''Abjad ``labeltools`` package.

Dependencies:

The ``labeltools`` package is a high-level labeling package.
The package may import essentially all of the core packages
at top level.
'''
from abjad.tools import systemtools

systemtools.ImportManager.import_structured_package(
	__path__[0],
	globals(),
	)

_documentation_section = 'core'



# -*- encoding: utf-8 -*-


def is_monotonically_decreasing_sequence(expr):
    r'''True when `expr` is a sequence and the elements in `expr` decrease monotonically:

    ::

        >>> expr = [9, 8, 7, 6, 5, 4, 3, 2, 1, 0]
        >>> sequencetools.is_monotonically_decreasing_sequence(expr)
        True

    ::

        >>> expr = [3, 3, 3, 3, 3, 3, 3, 2, 1, 0]
        >>> sequencetools.is_monotonically_decreasing_sequence(expr)
        True

    ::

        >>> expr = [3, 3, 3, 3, 3, 3, 3, 3, 3, 3]
        >>> sequencetools.is_monotonically_decreasing_sequence(expr)
        True

    False when `expr` is a sequence and the elements in `expr` do not decrease monotonically:

    ::

        >>> expr = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]
        >>> sequencetools.is_monotonically_decreasing_sequence(expr)
        False

    ::

        >>> expr = [0, 1, 2, 3, 3, 3, 3, 3, 3, 3]
        >>> sequencetools.is_monotonically_decreasing_sequence(expr)
        False

    True when `expr` is a sequence and `expr` is empty:

    ::

        >>> expr = []
        >>> sequencetools.is_monotonically_decreasing_sequence(expr)
        True

    False when `expr` is not a sequence:

    ::

        >>> sequencetools.is_monotonically_decreasing_sequence(17)
        False

    Returns boolean.
    '''

    try:
        previous = None
        for current in expr:
            if previous is not None:
                if not current <= previous:
                    return False
            previous = current
        return True

    except TypeError:
        return False



# -*- encoding: utf-8 -*-
from abjad import *


def test_lilypondfiletools_ScoreBlock_01():
    r'''Midi block is formatted when empty by default.
    Layout block must be explicitly set to format when empty.
    '''

    score = Score([Staff("c'8 d'8 e'8 f'8")])
    score_block = lilypondfiletools.ScoreBlock()
    layout_block = lilypondfiletools.LayoutBlock()
    layout_block.is_formatted_when_empty = True
    midi_block = lilypondfiletools.MIDIBlock()

    score_block.append(score)
    score_block.append(layout_block)
    score_block.append(midi_block)

    assert systemtools.TestManager.compare(
        score_block,
        r'''
        \score {
            \new Score <<
                \new Staff {
                    c'8
                    d'8
                    e'8
                    f'8
                }
            >>
            \layout {}
            \midi {}
        }
        '''
        )


def test_lilypondfiletools_ScoreBlock_02():
    r'''ScoreBlock does not format when empty by default.
    '''

    score_block = lilypondfiletools.ScoreBlock()
    assert format(score_block) == ''



# -*- encoding: utf-8 -*-
from abjad import *


def test_pitcharraytools_PitchArrayRow_to_measure_01():

    array = pitcharraytools.PitchArray([
        [1, (2, 1), ([-2, -1.5], 2)],
        [(7, 2), (6, 1), 1],
        ])

    '''
    [  ] [d'] [bf bqf     ]
    [g'      ] [fs'    ] []
    '''

    measure = array.rows[0].to_measure()

    r'''
    {
        \time 4/8
        r8
        d'8
        <bf bqf>4
    }
    '''

    assert inspect(measure).is_well_formed()
    assert systemtools.TestManager.compare(
        measure,
        r'''
        {
            \time 4/8
            r8
            d'8
            <bf bqf>4
        }
        '''
        )



# -*- encoding: utf-8 -*-
from abjad import *


def test_scoretools_Staff_engraver_removals_01():

    staff = Staff("c'8 d'8 e'8 f'8")
    staff.engraver_removals.append('Time_signature_engraver')
    staff.engraver_removals.append('Bar_number_engraver')

    r'''
    \new Staff \with {
        \remove Time_signature_engraver
        \remove Bar_number_engraver
    } {
        c'8
        d'8
        e'8
        f'8
    }
    '''

    assert inspect(staff).is_well_formed()
    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff \with {
            \remove Time_signature_engraver
            \remove Bar_number_engraver
        } {
            c'8
            d'8
            e'8
            f'8
        }
        '''
        )



# -*- encoding: utf-8 -*-
from abjad.tools import lilypondnametools
from abjad.tools.topleveltools import override
from abjad.tools.topleveltools import contextualize
from abjad.tools.lilypondfiletools.AttributedBlock import AttributedBlock


class ContextBlock(AttributedBlock):
    r'''A LilyPond input file context block.

    ::

        >>> context_block = lilypondfiletools.ContextBlock()

    ::

        >>> context_block
        ContextBlock()

    ::

        >>> context_block.context_name = 'Score'
        >>> override(context_block).bar_number.transparent = True
        >>> scheme = schemetools.Scheme('end-of-line-invisible')
        >>> override(context_block).time_signature.break_visibility = scheme
        >>> moment = schemetools.SchemeMoment((1, 45))
        >>> contextualize(context_block).proportionalNotationDuration = moment

    ..  doctest::

        >>> print format(context_block)
        \context {
            \Score
            \override BarNumber #'transparent = ##t
            \override TimeSignature #'break-visibility = #end-of-line-invisible
            proportionalNotationDuration = #(ly:make-moment 1 45)
        }

    '''

    ### INITIALIZER ###

    def __init__(self, context_name=None):
        AttributedBlock.__init__(self)
        self._accepts = []
        self._engraver_consists = []
        self._engraver_removals = []
        self._escaped_name = r'\context'
        self.alias = None
        self.context_name = context_name
        self.name = None
        self.type = None

    ### PRIVATE PROPERTIES ###

    @property
    def _format_pieces(self):
        from abjad.tools import systemtools
        result = []
        result.append('%s {' % self._escaped_name)
        manager = systemtools.LilyPondFormatManager
        # CAUTION: source context name must come before type to allow 
        # context redefinition.
        # TODO: rename self.context_name to self.source_context_name
        if self.context_name is not None:
            result.append('\t' + r'\%s' % self.context_name)
        if self.name is not None:
            result.append('\t' + r'\name %s' % self.name)
        if self.type is not None:
            result.append('\t' + r'\type %s' % self.type)
        if self.alias is not None:
            result.append('\t' + r'\alias %s' % self.alias)
        for string in self.engraver_removals:
            result.append('\t' + r'\remove %s' % string)
        # CAUTION: LilyPond consist statements are order-significant!
        for string in self.engraver_consists:
            result.append('\t' + r'\consists %s' % string)
        for string in self.accepts:
            result.append('\t' + r'\accepts %s' % string)
        for string in override(self)._list_format_contributions('override'):
            result.append('\t' + string)
        setting_contributions = []
        for key, value in contextualize(self)._get_attribute_tuples():
            setting_contribution = \
                manager.format_lilypond_context_setting_in_with_block(
                    key, value)
            setting_contributions.append(setting_contribution)
        for setting_contribution in sorted(setting_contributions):
            result.append('\t' + setting_contribution)
        result.append('}')
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def accepts(self):
        r'''Context accepts commands.

        Returns sets.
        '''
        return self._accepts

    @property
    def engraver_consists(self):
        r'''Engraver consists commands.

        Returns set.
        '''
        return self._engraver_consists

    @property
    def engraver_removals(self):
        r'''Engraver removal commands.

        Returns set.
        '''
        return self._engraver_removals

    ### PUBLIC PROPERTIES ###

    @property
    def alias(self):
        r'''Gets and sets alias of context block.

        Returns string or none.
        '''
        return self._alias

    @alias.setter
    def alias(self, alias):
        assert isinstance(alias, (str, type(None)))
        self._alias = alias

    @property
    def context_name(self):
        r'''Gets and sets context name of context block.

        Returns string or none.
        '''
        return self._context_name

    @context_name.setter
    def context_name(self, context_name):
        assert isinstance(context_name, (str, type(None)))
        self._context_name = context_name

    @property
    def name(self):
        r'''Gets and sets name of context block.

        Returns string or none.
        '''
        return self._name

    @name.setter
    def name(self, name):
        assert isinstance(name, (str, type(None)))
        self._name = name

    @property
    def type(self):
        r'''Gets and sets LilyPond type of context block.

        Returns string or none.
        '''
        return self._type

    @type.setter
    def type(self, expr):
        assert isinstance(expr, (str, type(None)))
        self._type = expr



# -*- encoding: utf-8 -*-
import argparse
import multiprocessing
import os
from abjad.tools import systemtools
from abjad.tools.developerscripttools.DirectoryScript import DirectoryScript


class PyTestScript(DirectoryScript):
    r'''Runs pytest on various Abjad paths.

    ..  shell::

        ajv test --help

    '''

    ### PUBLIC PROPERTIES ###

    @property
    def alias(self):
        r'''Alias of script.

        Returns ``'test'``.
        '''
        return 'test'

    @property
    def long_description(self):
        r'''Long description of script.

        Returns string or none.
        '''
        return None

    @property
    def scripting_group(self):
        r'''Scripting group of script.

        Returns none.
        '''
        return None

    @property
    def short_description(self):
        r'''Short description of script.

        Returns string.
        '''
        return 'Run "pytest" on various Abjad paths.'

    @property
    def version(self):
        r'''Version of script.

        Returns float.
        '''
        return 1.0

    ### PUBLIC METHODS ###

    def process_args(self, args):
        r'''Processes `args`.

        Returns none.
        '''
        import pytest
        parallel = ''
        if args.parallel:
            parallel = '-n {}'.format(multiprocessing.cpu_count())
        exitfirst = ''
        if args.exitfirst:
           exitfirst = '-x'
        report = ''
        if args.report:
            report = '-r {}'.format(args.report)
        print 'TESTING:'
        for path in args.path:
            print '\t{}'.format(path)
        print ''
        path = ' '.join(args.path)
        command = '{} {} {} {}'.format(parallel, exitfirst, report, path)
        return pytest.main(command.split())

    def setup_argument_parser(self, parser):
        r'''Sets up argument `parser`.

        Returns none.
        '''
        from abjad import abjad_configuration
        parser.add_argument('-p', '--parallel',
            action='store_true',
            dest='parallel',
            help='run pytest with multiprocessing',
            )
        parser.add_argument('-r', '--report',
            action='store',
            dest='report',
            help='show extra test summary info as specified by chars ' + \
                '(f)ailed, (E)error, (s)skipped, (x)failed, (X)passed.',
            metavar='chars',
            )
        parser.add_argument('-x', '--exitfirst',
            action='store_true',
            dest='exitfirst',
            help='stop on first failure',
            )
        group = parser.add_mutually_exclusive_group()
        group.add_argument('-A', '--all',
            action='store_const',
            const=[abjad_configuration.abjad_root_directory_path],
            dest='path',
            help='test all directories, including demos',
            )
        group.add_argument('-D', '--demos',
            action='store_const',
            const=[os.path.join(
                abjad_configuration.abjad_directory_path, 'demos')],
            dest='path',
            help='test demos directory',
            )
        group.add_argument('-M', '--mainline',
            action='store_const',
            const=[os.path.join(
                abjad_configuration.abjad_directory_path, 'tools')],
            dest='path',
            help='test mainline tools directory',
            )
        group.add_argument('-X', '--experimental',
            action='store_const',
            const=[abjad_configuration.abjad_experimental_directory_path],
            dest='path',
            help='test experimental directory',
            )
        parser.set_defaults(path=[
            os.path.join(abjad_configuration.abjad_directory_path, 'tools'),
            abjad_configuration.abjad_experimental_directory_path
            ])



# -*- encoding: utf-8 -*-
import re


space_delimited_lowercase_regex = re.compile('^(([a-z,0-9]+[ ]+)*[a-z,0-9]+)?$')

def is_space_delimited_lowercase_string(expr):
    r'''True when `expr` is a string and is space-delimited lowercase:

    ::

        >>> stringtools.is_space_delimited_lowercase_string('foo bar')
        True

    Otherwise false:

    ::

        >>> stringtools.is_space_delimited_lowercase_string('foo_bar')
        False

    Returns boolean.
    '''

    if not isinstance(expr, str):
        return False

    return bool(space_delimited_lowercase_regex.match(expr))



# -*- encoding: utf-8 -*-
import types
from abjad.tools.lilypondfiletools.AttributedBlock import AttributedBlock


class PaperBlock(AttributedBlock):
    r'''A LilyPond input file paper block:

    ::

        >>> paper_block = lilypondfiletools.PaperBlock()

    ::

        >>> paper_block
        PaperBlock()

    ::

        >>> paper_block.print_page_number = True
        >>> paper_block.print_first_page_number = False

    ..  doctest::

        >>> print format(paper_block)
        \paper {
            print-first-page-number = ##f
            print-page-number = ##t
        }

    '''

    ### INITIALIZER ###

    def __init__(self):
        AttributedBlock.__init__(self)
        self._escaped_name = r'\paper'
        self.minimal_page_breaking = None

    ### PRIVATE PROPERTIES ###

    @property
    def _formatted_user_attributes(self):
        result = []
        if self.minimal_page_breaking:
            result.append('#(define page-breaking ly:minimal-breaking)')
        result.extend(AttributedBlock._formatted_user_attributes.fget(self))
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def minimal_page_breaking(self):
        r'''Gets and sets minimal page-breaking flag on paper block.

        Returns boolean.
        '''
        return self._minimal_page_breaking

    @minimal_page_breaking.setter
    def minimal_page_breaking(self, expr):
        if isinstance(expr, (bool, type(None))):
            self._minimal_page_breaking = expr
        else:
            message = 'must be boolean or none.'
            raise TypeError(message)



# -*- encoding: utf-8 -*-
import copy
from abjad.tools import stringtools
from abjad.tools.abctools.AbjadObject import AbjadObject


class Articulation(AbjadObject):
    r'''An articulation.

    ..  container:: example

        Initializes from name:

        ::

            >>> Articulation('staccato')
            Articulation('staccato')

    ..  container:: example

        Initializes from abbreviation:

        ::

            >>> Articulation('.')
            Articulation('.')

    ..  container:: example

        Initializes from other articulation:

        ::

            >>> articulation = Articulation('staccato')
            >>> Articulation(articulation)
            Articulation('staccato')

    ..  container:: example

        Initializes with direction:

        ::

            >>> Articulation('staccato', Up)
            Articulation('staccato', Up)

    .. container:: example

        Use `attach()` to attach articulations to notes, rests or chords:

        ::

            >>> note = Note("c'4")
            >>> articulation = Articulation('staccato')
            >>> attach(articulation, note)
            >>> show(note) # doctest: +SKIP

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_name',
        '_direction',
        '_format_slot',
        )

    ### INITIALIZER ###

    def __init__(self, *args):
        from abjad.tools import datastructuretools
        assert len(args) in range(3), repr(args)
        if 2 <= len(args):
            assert isinstance(args[0], (str, type(None))), repr(args[0])
            assert isinstance(args[1],
                (str, type(None), datastructuretools.OrdinalConstant)), \
                repr(args[1])
            name, direction = args
        elif len(args) == 1 and isinstance(args[0], type(self)):
            name = args[0].name
            direction = args[0].direction
        elif len(args) == 1:
            assert isinstance(args[0],
                (str, type(None), datastructuretools.OrdinalConstant)), \
                repr(args[0])
            if args[0]:
                splits = args[0].split('\\')
                assert len(splits) in (1, 2), repr(splits)
                if len(splits) == 1:
                    name, direction = args[0], None
                elif len(splits) == 2:
                    name = splits[1]
                    if splits[0]:
                        direction = splits[0]
                    else:
                        direction = None
            else:
                name, direction = None, None
        else:
            name, direction = None, None
        self._name = name
        direction = \
            stringtools.arg_to_tridirectional_ordinal_constant(direction)
        self._direction = direction
        self._format_slot = 'right'

    ### SPECIAL METHODS ###

    def __copy__(self, *args):
        r'''Copies articulation.

        Returns new articulation.
        '''
        return type(self)(self.name, self.direction)

    def __eq__(self, expr):
        r'''True when `expr` is an articulation with name and direction 
        equal to that of this articulation. Otherwise false.

        Returns boolean.
        '''
        if isinstance(expr, type(self)):
            if expr.name == self.name:
                if self.direction == expr.direction:
                    return True
        return False

    def __format__(self, format_specification=''):
        r'''Formats articulation.

        Set `format_specification` to `''`, `'lilypond`' or `'storage'`.
        Interprets `''` equal to `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatManager.get_storage_format(self)
        elif format_specification == 'lilypond':
            return self._lilypond_format
        return str(self)

    def __illustrate__(self):
        r'''Illustrates articulation.

        Returns LilyPond file.
        '''
        from abjad.tools import lilypondfiletools
        from abjad.tools import markuptools
        from abjad.tools import scoretools
        from abjad.tools import topleveltools
        note = scoretools.Note("c'4")
        articulation = copy.copy(self)
        topleveltools.attach(articulation, note)
        lilypond_file = lilypondfiletools.make_basic_lilypond_file(note)
        lilypond_file.header_block.tagline = markuptools.Markup('""')
        return lilypond_file

    def __str__(self):
        r'''String representation of articulation.

        Returns string.
        '''
        if self.name:
            string = self._shortcut_to_word.get(self.name)
            if not string:
                string = self.name
            if self.direction is None:
                direction = '-'
            else:
                direction = \
                    stringtools.arg_to_tridirectional_lilypond_symbol(
                    self.direction)
            return '%s\%s' % (direction, string)
        else:
            return ''

    ### PRIVATE PROPERTIES ###

    # this causes unnecessary coupling to changeable lilypond codebase
    # and is discouraged
    _articulations_supported = (
        'accent',
        'marcato',
        'staccatissimo',
        'espressivo'
        'staccato',
        'tenuto'
        'portato'
        'upbow',
        'downbow'
        'flageolet'
        'thumb',
        'lheel'
        'rheel'
        'ltoe',
        'rtoe'
        'open'
        'stopped',
        'turn'
        'reverseturn'
        'trill',
        'prall'
        'mordent'
        'prallprall'
        'prallmordent',
        'upprall',
        'downprall',
        'upmordent',
        'downmordent',
        'pralldown',
        'prallup',
        'lineprall',
        'signumcongruentiae',
        'shortfermata',
        'fermata',
        'longfermata',
        'verylongfermata',
        'segno',
        'coda',
        'varcoda',
        '^',
        '+',
        '-',
        '|',
        '>',
        '.',
        '_',
        )

    # this causes unnecessary coupling to changeable lilypond codebase
    # and is discouraged
    _shortcut_to_word = {
        '^': 'marcato',
        '+': 'stopped',
        '-': 'tenuto',
        '|': 'staccatissimo',
        '>': 'accent',
        '.': 'staccato',
        '_': 'portato',
        }

    @property
    def _contents_repr_string(self):
        if self.direction is not None:
            return '{!r}, {!r}'.format(self.name, self.direction)
        else:
            return repr(self.name)

    @property
    def _lilypond_format(self):
        return str(self)

    @property
    def _repr_specification(self):
        from abjad.tools import systemtools
        positional_argument_values = [self.name]
        if self.direction is not None:
            positional_argument_values.append(self.direction)
        return systemtools.StorageFormatSpecification(
            self,
            is_indented=False,
            keyword_argument_names=(),
            positional_argument_values=positional_argument_values,
            )

    ### PUBLIC PROPERTIES ###

    @property
    def direction(self):
        r'''Direction of articulation.

        Returns ordinal constant or none.
        '''
        assert self._direction in (Up, Down, Center, None), \
            repr(self._direction)
        return self._direction


    @property
    def name(self):
        r'''Name of articulation.

        ::

            >>> articulation.name
            'staccato'

        Returns string.
        '''
        return self._name



# -*- encoding: utf-8 -*-
from abjad.tools import schemetools
from abjad.tools.abctools import AbjadObject


class MarkupCommand(AbjadObject):
    r'''A LilyPond markup command.

    ::

        >>> circle = markuptools.MarkupCommand('draw-circle', 2.5, 0.1, False)
        >>> square = markuptools.MarkupCommand('rounded-box', 'hello?')
        >>> line = markuptools.MarkupCommand('line', [square, 'wow!'])
        >>> rotate = markuptools.MarkupCommand('rotate', 60, line)
        >>> combine = markuptools.MarkupCommand('combine', rotate, circle)

    ::

        >>> print format(combine, 'lilypond')
        \combine
            \rotate
                #60
                \line
                    {
                        \rounded-box
                            hello?
                        wow!
                    }
            \draw-circle
                #2.5
                #0.1
                ##f

    Insert a markup command in markup in order to attach it to 
    score components:

    ::

        >>> note = Note("c'4")
        >>> markup = markuptools.Markup(combine)
        >>> attach(markup, note)
        >>> show(note) # doctest: +SKIP

    ..  doctest::

        >>> print format(note)
        c'4
            - \markup {
                \combine
                    \rotate
                        #60
                        \line
                            {
                                \rounded-box
                                    hello?
                                wow!
                            }
                    \draw-circle
                        #2.5
                        #0.1
                        ##f
                }

    Markup commands are immutable.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_args', 
        '_command',
        )

    ### INITIALIZER ###

    def __init__(self, command=None, *args):
        if command is None:
            command = 'draw-circle'
            assert len(args) == 0
        assert isinstance(command, str) \
            and len(command) and command.find(' ') == -1
        self._command = command
        self._args = tuple(args)

    ### SPECIAL METHODS ###

    def __eq__(self, expr):
        r'''True when `expr` is a markup command with command and args equal to
        those of this markup command. Otherwise false.

        Returns boolean.
        '''
        if isinstance(expr, type(self)):
            if self.command == expr.command:
                if self.args == expr.args:
                    return True
        return False

    def __format__(self, format_specification=''):
        r'''Formats markup command.

        Set `format_specification` to `''`, `'lilypond'` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatManager.get_storage_format(self)
        elif format_specification == 'lilypond':
            return self._lilypond_format
        return str(self)

    def __str__(self):
        r'''String representation of markup command.

        Returns string.
        '''
        return self._lilypond_format

    ### PRIVATE PROPERTIES ###

    @property
    def _lilypond_format(self):
        return '\n'.join(self._get_format_pieces())

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        positional_argument_values = (self.command,) + self.args
        return systemtools.StorageFormatSpecification(
            self,
            keyword_argument_names=(),
            positional_argument_values=positional_argument_values,
            )

    ### PRIVATE METHODS ###

    def _escape_string(self, string):
        if -1 == string.find(' '):
            return string
        string = repr(string)
        if string.startswith("'") and string.endswith("'"):
            string = string.replace('"', '\"')
            string = '"' + string[1:]
            string = string[:-1] + '"'
        return string

    def _get_format_pieces(self):
        indent = '\t'
        def recurse(iterable):
            result = []
            for x in iterable:
                if isinstance(x, (list, tuple)):
                    result.append('{')
                    result.extend(recurse(x))
                    result.append('}')
                elif isinstance(x, type(self)):
                    result.extend(x._get_format_pieces())
                elif isinstance(x, schemetools.Scheme):
                    result.append(format(x))
                else:
                    formatted = schemetools.Scheme.format_scheme_value(x)
                    if isinstance(x, str):
                        result.append(formatted)
                    else:
                        result.append('#{}'.format(formatted))
            return ['{}{}'.format(indent, x) for x in result]
        parts = [r'\{}'.format(self.command)]
        parts.extend(recurse(self.args))
        return parts

    ### PUBLIC PROPERTIES ###

    @property
    def args(self):
        r'''Markup command arguments.

        Returns tuple.
        '''
        return self._args

    # TODO: change to MarkupCommand.name
    @property
    def command(self):
        r'''Markup command name.

        Returns string.
        '''
        return self._command



# -*- encoding: utf-8 -*-
from abjad import *


def test_indicatortools_TempoInventory___format___01():

    tempo_inventory = indicatortools.TempoInventory([
        ('Allegro', (1, 4), 84),
        ])

    assert systemtools.TestManager.compare(
        format(tempo_inventory),
        r'''
        indicatortools.TempoInventory(
            [
                indicatortools.Tempo(
                    'Allegro',
                    durationtools.Duration(1, 4),
                    84
                    ),
                ]
            )
        ''',
        )



# -*- encoding: utf-8 -*-
import abc
from abjad.tools.abctools.AbjadObject import AbjadObject


class TimeRelation(AbjadObject):
    r'''A time relation.

    Time relations are immutable.
    '''

    ### INITIALIZER ###

    def __init__(self, inequality=None):
        from abjad.tools import timespantools
        default_inequality = timespantools.CompoundInequality([
            'timespan_1.start_offset < timespan_2.start_offset',
            ])
        inequality = inequality or default_inequality
        assert isinstance(
            inequality, timespantools.CompoundInequality), repr(inequality)
        self._inequality = inequality

    ### SPECIAL METHODS ###

    @abc.abstractmethod
    def __call__(self):
        r'''Evaluates time relation.

        Returns boolean.
        '''
        pass

    @abc.abstractmethod
    def __eq__(self, expr):
        r'''True when `expr` is a equal-valued time relation.
        Otherwise false.

        Returns boolean.
        '''
        pass

    def __format__(self, format_specification=''):
        r'''Formats time relation.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatManager.get_storage_format(self)
        return str(self)

    ### PUBLIC PROPERTIES ###

    @property
    def inequality(self):
        r'''Time relation inequality.

        Return ineqality.
        '''
        return self._inequality

    @abc.abstractproperty
    def is_fully_loaded(self):
        r'''True when both time relation terms are not none.
        Otherwise false:

        Returns boolean.
        '''
        pass

    @abc.abstractproperty
    def is_fully_unloaded(self):
        r'''True when both time relation terms are none.
        Otherwise false:

        Returns boolean.
        '''
        pass

    ### PUBLIC METHODS ###

    def __makenew__(self, *args, **kwargs):
        r'''Makes new time relation with optional `args` and `kwargs`.

        Returns new time relation.
        '''
        from abjad.tools import systemtools
        manager = systemtools.StorageFormatManager
        keyword_argument_dictionary = \
            manager.get_keyword_argument_dictionary(self)
        for key, value in kwargs.iteritems():
            keyword_argument_dictionary[key] = value
        result = type(self)(
            *manager.get_positional_argument_values(self),
            **keyword_argument_dictionary
            )
        return result



# -*- encoding: utf-8 -*-
from abjad import *


def test_pitchtools_apply_accidental_to_named_pitch_01():

    assert pitchtools.apply_accidental_to_named_pitch(
        NamedPitch('cs', 4), 'sharp') == NamedPitch('css', 4)
    assert pitchtools.apply_accidental_to_named_pitch(
        NamedPitch('cs', 4), 'flat') == NamedPitch('c', 4)
    assert pitchtools.apply_accidental_to_named_pitch(
        NamedPitch('cs', 4), 'natural') == NamedPitch('cs', 4)
    assert pitchtools.apply_accidental_to_named_pitch(
        NamedPitch('cs', 4), 'quarter sharp') == NamedPitch('ctqs', 4)



# -*- encoding: utf-8 -*-
from abjad import *
from abjad.tools import sequencetools


def test_sequencetools_all_are_nonnegative_integers_01():

    assert sequencetools.all_are_nonnegative_integers([0, 1, 2, 3, 99])


def test_sequencetools_all_are_nonnegative_integers_02():

    assert not sequencetools.all_are_nonnegative_integers([-1, 2, 3, 99])
    assert not sequencetools.all_are_nonnegative_integers(7)
    assert not sequencetools.all_are_nonnegative_integers('foo')



# -*- encoding: utf-8 -*-


def iterate_sequence_nwise_strict(sequence, n):
    '''Iterate elements in `sequence` `n` at a time:

    ::

        >>> for x in sequencetools.iterate_sequence_nwise_strict(range(10), 4):
        ...     x
        ...
        (0, 1, 2, 3)
        (1, 2, 3, 4)
        (2, 3, 4, 5)
        (3, 4, 5, 6)
        (4, 5, 6, 7)
        (5, 6, 7, 8)
        (6, 7, 8, 9)

    Returns generator.
    '''

    element_buffer = []
    for element in sequence:
        element_buffer.append(element)
        if len(element_buffer) == n:
            yield tuple(element_buffer)
            element_buffer.pop(0)



# -*- encoding: utf-8 -*-
from abjad.tools.abctools import AbjadObject


class IsUnpitched(AbjadObject):
    r'''Is unpitched indicator.

    ::

        >>> indicator = indicatortools.IsUnpitched()

    Attach to score selection to denote unpitched music.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )



# -*- encoding: utf-8 -*-
from abjad import *
import pytest


def test_scoretools_Measure_extend_01():
    r'''Do not adjust time signature automatically.
    '''

    measure = Measure((3, 4), "c' d' e'")
    measure.extend("f' g'")

    assert measure.is_overfull
    assert pytest.raises(Exception, 'f(measure)')


def test_scoretools_Measure_extend_02():
    r'''Adjust time signature automatically.
    '''

    measure = Measure((3, 4), "c' d' e'")
    measure.automatically_adjust_time_signature = True
    measure.extend("f' g'")

    assert not measure.is_misfilled
    assert systemtools.TestManager.compare(
        measure,
        r'''
        {
            \time 5/4
            c'4
            d'4
            e'4
            f'4
            g'4
        }
        '''
        )



# -*- encoding: utf-8 -*-
from abjad import *


def test_spannertools_Decrescendo___init___01():
    r'''Initialize empty decrescendo spanner.
    '''

    decrescendo = Decrescendo()
    assert isinstance(decrescendo, Decrescendo)


def test_spannertools_Decrescendo___init___02():

    staff = Staff("c'8 d'8 e'8 f'8 g'2")
    decrescendo = Decrescendo()
    attach(decrescendo, staff[:4])

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            c'8 \>
            d'8
            e'8
            f'8 \!
            g'2
        }
        '''
        )



# -*- encoding: utf-8 -*-
import pytest
from abjad import *


def test_spannertools_PianoPedalSpanner_01():

    staff = Staff(scoretools.make_repeated_notes(4))
    piano_pedal_spanner = spannertools.PianoPedalSpanner()
    attach(piano_pedal_spanner, staff[:])

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            \set Staff.pedalSustainStyle = #'mixed
            c'8 \sustainOn
            c'8
            c'8
            c'8 \sustainOff
        }
        '''
        )

    assert inspect(staff).is_well_formed()
    assert piano_pedal_spanner.kind == 'sustain'
    assert piano_pedal_spanner.style == 'mixed'


def test_spannertools_PianoPedalSpanner_02():
    r'''Piano pedal spanner supports sostenuto pedal.
    '''

    staff = Staff(scoretools.make_repeated_notes(4))
    piano_pedal_spanner = spannertools.PianoPedalSpanner()
    piano_pedal_spanner.kind = 'sostenuto'
    attach(piano_pedal_spanner, staff[:])

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            \set Staff.pedalSustainStyle = #'mixed
            c'8 \sostenutoOn
            c'8
            c'8
            c'8 \sostenutoOff
        }
        '''
        )

    assert inspect(staff).is_well_formed()


def test_spannertools_PianoPedalSpanner_03():
    r'''Piano pedal spanner supports una corda pedal.
    '''

    staff = Staff(scoretools.make_repeated_notes(4))
    piano_pedal_spanner = spannertools.PianoPedalSpanner()
    piano_pedal_spanner.kind = 'corda'
    attach(piano_pedal_spanner, staff[:])

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            \set Staff.pedalSustainStyle = #'mixed
            c'8 \unaCorda
            c'8
            c'8
            c'8 \treCorde
        }
        '''
        )

    assert inspect(staff).is_well_formed()


def test_spannertools_PianoPedalSpanner_04():
    r'''PianoPedal spanner supports text style.
    '''

    staff = Staff(scoretools.make_repeated_notes(4))
    piano_pedal_spanner = spannertools.PianoPedalSpanner()
    attach(piano_pedal_spanner, staff[:])
    assert piano_pedal_spanner.kind == 'sustain'
    piano_pedal_spanner.style = 'text'

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            \set Staff.pedalSustainStyle = #'text
            c'8 \sustainOn
            c'8
            c'8
            c'8 \sustainOff
        }
        '''
        )

    assert inspect(staff).is_well_formed()


def test_spannertools_PianoPedalSpanner_05():
    r'''PianoPedal spanner supports bracket style.
    '''

    staff = Staff(scoretools.make_repeated_notes(4))
    piano_pedal_spanner = spannertools.PianoPedalSpanner()
    attach(piano_pedal_spanner, staff[:])
    assert piano_pedal_spanner.kind == 'sustain'
    piano_pedal_spanner.style = 'bracket'

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            \set Staff.pedalSustainStyle = #'bracket
            c'8 \sustainOn
            c'8
            c'8
            c'8 \sustainOff
        }
        '''
        )

    assert inspect(staff).is_well_formed()


def test_spannertools_PianoPedalSpanner_06():
    r'''Consecutive dovetailing PianoPedal spanners format correctly.
    '''

    staff = Staff(scoretools.make_repeated_notes(8))
    piano_pedal_spanner = spannertools.PianoPedalSpanner()
    attach(piano_pedal_spanner, staff[:4])
    piano_pedal_spanner = spannertools.PianoPedalSpanner()
    attach(piano_pedal_spanner, staff[3:])

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            \set Staff.pedalSustainStyle = #'mixed
            c'8 \sustainOn
            c'8
            c'8
            \set Staff.pedalSustainStyle = #'mixed
            c'8 \sustainOff \sustainOn
            c'8
            c'8
            c'8
            c'8 \sustainOff
        }
        '''
        )

    assert inspect(staff).is_well_formed()


def test_spannertools_PianoPedalSpanner_07():
    r'''The 'kind' and 'style' attributes raise ValueError as needed.
    '''

    staff = Staff(scoretools.make_repeated_notes(4))
    piano_pedal_spanner = spannertools.PianoPedalSpanner()
    attach(piano_pedal_spanner, staff[:])

    assert pytest.raises(ValueError, 'piano_pedal_spanner.kind = "abc"')
    assert pytest.raises(ValueError, 'piano_pedal_spanner.style = "abc"')



# -*- encoding: utf-8 -*-
from abjad import *


def create_pitch_contour_reservoir():
    r'''Creates pitch contour reservoir.
    '''

    scale = tonalanalysistools.Scale('a', 'minor')
    pitch_ranges = {
        'First Violin': pitchtools.PitchRange(("c'", "a'''")),
        'Second Violin': pitchtools.PitchRange(('a', "a''")),
        'Viola': pitchtools.PitchRange(('e', "a'")),
        'Cello': pitchtools.PitchRange(('a,', 'a')),
        'Bass': pitchtools.PitchRange(('c', 'a')),
    }

    reservoir = {}
    for instrument_name, pitch_range in pitch_ranges.iteritems():
        pitch_set = scale.create_named_pitch_set_in_pitch_range(pitch_range)
        pitches = sorted(pitch_set, reverse=True)
        pitch_descents = []
        for i in xrange(len(pitches)):
            descent = tuple(pitches[:i + 1])
            pitch_descents.append(descent)
        reservoir[instrument_name] = tuple(pitch_descents)

    return reservoir



# -*- encoding: utf-8 -*-
import pytest
from abjad import *


def test_sequencetools_get_sequence_period_of_rotation_01():

    assert sequencetools.get_sequence_period_of_rotation([1, 1, 1, 1, 1, 1], 1) == 1
    assert sequencetools.get_sequence_period_of_rotation([1, 2, 1, 2, 1, 2], 1) == 2
    assert sequencetools.get_sequence_period_of_rotation([1, 2, 1, 1, 2, 1], 1) == 3
    assert sequencetools.get_sequence_period_of_rotation([1, 2, 1, 1, 1, 1], 1) == 6


def test_sequencetools_get_sequence_period_of_rotation_02():

    assert sequencetools.get_sequence_period_of_rotation([1, 1, 1, 1, 1, 1], 2) == 1
    assert sequencetools.get_sequence_period_of_rotation([1, 2, 1, 2, 1, 2], 2) == 1
    assert sequencetools.get_sequence_period_of_rotation([1, 2, 1, 1, 2, 1], 2) == 3
    assert sequencetools.get_sequence_period_of_rotation([1, 2, 1, 1, 1, 1], 2) == 3


def test_sequencetools_get_sequence_period_of_rotation_03():

    assert sequencetools.get_sequence_period_of_rotation([1, 1, 1, 1, 1, 1], 3) == 1
    assert sequencetools.get_sequence_period_of_rotation([1, 2, 1, 2, 1, 2], 3) == 2
    assert sequencetools.get_sequence_period_of_rotation([1, 2, 1, 1, 2, 1], 3) == 1
    assert sequencetools.get_sequence_period_of_rotation([1, 2, 1, 1, 1, 1], 3) == 2


def test_sequencetools_get_sequence_period_of_rotation_04():

    assert sequencetools.get_sequence_period_of_rotation([1, 1, 1, 1, 1, 1], 10) == 1
    assert sequencetools.get_sequence_period_of_rotation([1, 2, 1, 2, 1, 2], 10) == 1
    assert sequencetools.get_sequence_period_of_rotation([1, 2, 1, 1, 2, 1], 10) == 3
    assert sequencetools.get_sequence_period_of_rotation([1, 2, 1, 1, 1, 1], 10) == 3


def test_sequencetools_get_sequence_period_of_rotation_05():
    r'''Empty iterable boundary case.
    '''

    statement = 'sequencetools.get_sequence_period_of_rotation([], 1)'
    assert pytest.raises(ZeroDivisionError, statement)



# -*- encoding: utf-8 -*-
from abjad import *


def test_sequencetools_iterate_sequence_nwise_strict_01():

    sequence_2 = list(sequencetools.iterate_sequence_nwise_strict(range(10), 3))
    assert sequence_2 == [(0, 1, 2), (1, 2, 3), (2, 3, 4), (3, 4, 5),
        (4, 5, 6), (5, 6, 7), (6, 7, 8), (7, 8, 9)]


def test_sequencetools_iterate_sequence_nwise_strict_02():
    r'''Empty iterable boundary case.
    '''

    empty = list(sequencetools.iterate_sequence_nwise_strict([], 3))
    assert empty == []


def test_sequencetools_iterate_sequence_nwise_strict_03():
    r'''Length-1 boundary case.
    '''

    sequence_2 = list(sequencetools.iterate_sequence_nwise_strict(range(10), 1))
    assert sequence_2 == [(0,), (1,), (2,), (3,), (4,), (5,), (6,), (7,), (8,), (9,)]



# -*- encoding: utf-8 -*-
from abjad.tools.lilypondfiletools.AttributedBlock import AttributedBlock


class HeaderBlock(AttributedBlock):
    r'''Abjad model of LilyPond input file header block:

    ::

        >>> header_block = lilypondfiletools.HeaderBlock()

    ::

        >>> header_block
        HeaderBlock()

    ::

        >>> header_block.composer = markuptools.Markup('Josquin')
        >>> header_block.title = markuptools.Markup('Missa sexti tonus')

    ..  doctest::

        >>> print format(header_block)
        \header {
            composer = \markup { Josquin }
            title = \markup { Missa sexti tonus }
        }

    Returns header block.
    '''

    def __init__(self):
        AttributedBlock.__init__(self)
        self._escaped_name = r'\header'



# -*- encoding: utf-8 -*-
from abjad import *
from abjad.tools.mathtools import NonreducedFraction


def test_mathtools_NonreducedFraction___sub___01():

    result = NonreducedFraction(5, 4) - NonreducedFraction(18, 16)
    assert result.pair == (2, 16)

    result = NonreducedFraction(18, 16) - NonreducedFraction(5, 4)
    assert result.pair == (-2, 16)


def test_mathtools_NonreducedFraction___sub___02():

    result = NonreducedFraction(3, 3) - 2
    assert result.pair == (-3, 3)

    result = 2 - NonreducedFraction(3, 3)
    assert result.pair == (3, 3)



# -*- encoding: utf-8 -*-
from abjad import *


def test_markuptools_make_big_centered_page_number_markup_01():

    markup = markuptools.make_big_centered_page_number_markup()

    assert systemtools.TestManager.compare(
        format(markup, 'lilypond'),
        r'''
        \markup {
            \fill-line
                {
                    \bold
                        \fontsize
                            #3
                            \concat
                                {
                                    \on-the-fly
                                        #print-page-number-check-first
                                        \fromproperty
                                            #'page:page-number-string
                                }
                }
            }
        ''',
        )



# -*- encoding: utf-8 -*-
from abjad.tools import scoretools


# TODO: remove in favor of layouttools.set_line_breaks_by_line_duration()
def set_line_breaks_cyclically_by_line_duration_ge(
    expr,
    line_duration,
    line_break_class=None,
    add_empty_bars=False,
    ):
    r'''Iterate `line_break_class` instances in `expr` and 
    accumulate duration.

    Add line break after every total less than or equal to `line_duration`:

    ::

        >>> staff = Staff()
        >>> staff.append(Measure((2, 8), "c'8 d'8"))
        >>> staff.append(Measure((2, 8), "e'8 f'8"))
        >>> staff.append(Measure((2, 8), "g'8 a'8"))
        >>> staff.append(Measure((2, 8), "b'8 c''8"))
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> print format(staff)
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
            {
                b'8
                c''8
            }
        }

    ::

        >>> layouttools.set_line_breaks_cyclically_by_line_duration_ge(
        ...     staff, 
        ...     Duration(4, 8),
        ...     )
        >>> show(staff) # doctest: +SKIP

    ::

        >>> print format(staff)
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
                \break
            }
            {
                g'8
                a'8
            }
            {
                b'8
                c''8
                \break
            }
        }

    When ``line_break_class=None`` set `line_break_class` to measure.
    '''
    from abjad.tools import layouttools

    if line_break_class is None:
        line_break_class = scoretools.Measure

    layouttools.set_line_breaks_by_line_duration(
        expr,
        line_duration,
        line_break_class,
        'prolated',
        add_empty_bars=add_empty_bars,
        )



# -*- encoding: utf-8 -*-
import re


lowercamelcase_regex = re.compile('^([a-z,0-9]+([A-Z,0-9]+[a-z,0-9]*)*)?$')

def is_lower_camel_case_string(expr):
    r'''True when `expr` is a string and is lowercamelcase:

    ::

        >>> stringtools.is_lower_camel_case_string('fooBar')
        True

    Otherwise false:

    ::

        >>> stringtools.is_lower_camel_case_string('FooBar')
        False

    Returns boolean.
    '''

    if not isinstance(expr, str):
        return False

    return bool(lowercamelcase_regex.match(expr))



# -*- encoding: utf-8 -*-
from abjad import *


def test_labeltools_label_leaves_in_expr_with_leaf_indices_01():
    r'''Leaf indices start at 0.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    labeltools.label_leaves_in_expr_with_leaf_indices(staff)

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            c'8
                _ \markup {
                    \small
                        0
                    }
            d'8
                _ \markup {
                    \small
                        1
                    }
            e'8
                _ \markup {
                    \small
                        2
                    }
            f'8
                _ \markup {
                    \small
                        3
                    }
        }
        '''
        )

    assert inspect(staff).is_well_formed()



# -*- encoding: utf-8 -*-
from abjad.tools import mathtools


def inventory_aggregate_subsets():
    '''Inventory aggregate subsets:

    ::

        >>> U_star = pitchtools.inventory_aggregate_subsets()
        >>> len(U_star)
        4096
        >>> for pcset in U_star[:20]:
        ...   pcset
        PitchClassSet([])
        PitchClassSet([0])
        PitchClassSet([1])
        PitchClassSet([0, 1])
        PitchClassSet([2])
        PitchClassSet([0, 2])
        PitchClassSet([1, 2])
        PitchClassSet([0, 1, 2])
        PitchClassSet([3])
        PitchClassSet([0, 3])
        PitchClassSet([1, 3])
        PitchClassSet([0, 1, 3])
        PitchClassSet([2, 3])
        PitchClassSet([0, 2, 3])
        PitchClassSet([1, 2, 3])
        PitchClassSet([0, 1, 2, 3])
        PitchClassSet([4])
        PitchClassSet([0, 4])
        PitchClassSet([1, 4])
        PitchClassSet([0, 1, 4])

    There are 4096 subsets of the aggregate.

    This is ``U*`` in [Morris 1987].

    Returns list of numbered pitch-class sets.
    '''
    from abjad.tools import pitchtools

    def _helper(binary_string):
        result = zip(binary_string, range(len(binary_string)))
        result = [x[1] for x in result if x[0] == '1']
        return result

    result = []

    for x in range(4096):
        subset = ''.join(list(reversed(mathtools.integer_to_binary_string(x).zfill(12))))
        subset = _helper(subset)
        subset = pitchtools.PitchClassSet(
            subset,
            item_class=pitchtools.NumberedPitchClass,
            )
        result.append(subset)

    return result



# -*- encoding: utf-8 -*-
import math


def binomial_coefficient(n, k):
    r'''Binomial coefficient of `n` choose `k`.

    ::

        >>> for k in range(8):
        ...     print k, '\t', mathtools.binomial_coefficient(8, k)
        ...
        0  1
        1  8
        2  28
        3  56
        4  70
        5  56
        6  28
        7  8

    Returns positive integer.
    '''

    return math.factorial(n) / (math.factorial(n - k) * math.factorial(k))



# -*- encoding: utf-8 -*-
import pytest
from abjad import *


def test_markuptools_MarkupCommand___init___01():
    '''Command must be a nonempty string without spaces.
    '''

    assert pytest.raises(Exception, "markuptools.MarkupCommand('')")
    assert pytest.raises(Exception, "markuptools.MarkupCommand(3.14159)")
    assert pytest.raises(Exception, "markuptools.MarkupCommand('one two')")



# -*- encoding: utf-8 -*-


def arg_to_bidirectional_direction_string(arg):
    r'''Convert `arg` to bidirectional direction string:

        >>> stringtools.arg_to_bidirectional_direction_string('^')
        'up'

    ::

        >>> stringtools.arg_to_bidirectional_direction_string('_')
        'down'

    ::

        >>> stringtools.arg_to_bidirectional_direction_string(1)
        'up'

    ::

        >>> stringtools.arg_to_bidirectional_direction_string(-1)
        'down'

    If `arg` is 'up' or 'down', `arg` will be returned.

    Returns string or none.
    '''

    lookup = {
        1: 'up',
        -1: 'down',
        Up: 'up',
        Down: 'down',
        '^': 'up',
        '_': 'down',
        'up': 'up',
        'down': 'down'
    }

    if arg in lookup:
        return lookup[arg]
    raise ValueError



# -*- encoding: utf-8 -*-
from abjad import *
import pytest


def test_scoretools_Leaf_get_duration_01():
    r'''Clock duration equals duration divide by effective tempo.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    tempo = Tempo(Duration(1, 4), 38)
    attach(tempo, staff)
    tempo = Tempo(Duration(1, 4), 42)
    attach(tempo, staff[2])
    Score([staff])

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            \tempo 4=38
            c'8
            d'8
            \tempo 4=42
            e'8
            f'8
        }
        '''
        )

    assert inspect(staff[0]).get_duration(in_seconds=True) == Duration(15, 19)
    assert inspect(staff[1]).get_duration(in_seconds=True) == Duration(15, 19)
    assert inspect(staff[2]).get_duration(in_seconds=True) == Duration(5, 7)
    assert inspect(staff[3]).get_duration(in_seconds=True) == Duration(5, 7)


def test_scoretools_Leaf_get_duration_02():
    r'''Clock duration can not calculate without tempo.
    '''

    note = Note("c'4")
    statement = 'inspect(note).get_duration(in_seconds=True)'
    assert pytest.raises(Exception, statement)



# -*- encoding: utf-8 -*-
from abjad import *


def test_spannertools_DynamicTextSpanner_01():

    voice = Voice("c'8 d'8 e'8 f'8")
    beam = Beam()
    attach(beam, voice[:])
    spanner = spannertools.DynamicTextSpanner(dynamic='f')
    attach(spanner, voice[:2])
    spanner = spannertools.DynamicTextSpanner(dynamic='p')
    attach(spanner, voice[2:])

    assert systemtools.TestManager.compare(
        voice,
        r'''
        \new Voice {
            c'8 [ \f
            d'8
            e'8 \p
            f'8 ]
        }
        '''
        )

    assert inspect(voice).is_well_formed()



# -*- encoding: utf-8 -*-
from abjad import *
from abjad.tools import sequencetools
from abjad.tools import rhythmmakertools


def test_rhythmmakertools_OutputBurnishedTaleaRhythmMaker___call___01():

    talea, talea_denominator, prolation_addenda = [1], 16, [2]
    lefts, middles, rights = [0], [-1], [0]
    left_lengths, right_lengths = [1], [1]
    maker = rhythmmakertools.OutputBurnishedTaleaRhythmMaker(
        talea, talea_denominator, prolation_addenda,
        lefts, middles, rights, left_lengths, right_lengths)

    divisions = [(3, 16), (3, 8)]
    music = maker(divisions)

    music = sequencetools.flatten_sequence(music)
    staff = Staff(scoretools.make_spacer_skip_measures(divisions))
    scoretools.replace_contents_of_measures_in_expr(staff, music)

    r'''
    \new Staff {
        {
            \time 3/16
            \tweak #'text #tuplet-number::calc-fraction-text
            \times 3/5 {
                c'16
                r16
                r16
                r16
                r16
            }
        }
        {
            \time 3/8
            \tweak #'text #tuplet-number::calc-fraction-text
            \times 3/4 {
                r16
                r16
                r16
                r16
                r16
                r16
                r16
                c'16
            }
        }
    }
    '''

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 3/16
                \tweak #'text #tuplet-number::calc-fraction-text
                \times 3/5 {
                    c'16
                    r16
                    r16
                    r16
                    r16
                }
            }
            {
                \time 3/8
                \tweak #'text #tuplet-number::calc-fraction-text
                \times 3/4 {
                    r16
                    r16
                    r16
                    r16
                    r16
                    r16
                    r16
                    c'16
                }
            }
        }
        '''
        )


def test_rhythmmakertools_OutputBurnishedTaleaRhythmMaker___call___02():

    talea, talea_denominator, prolation_addenda = [1], 4, [2]
    lefts, middles, rights = [-1], [0], [-1]
    left_lengths, right_lengths = [1], [1]
    maker = rhythmmakertools.OutputBurnishedTaleaRhythmMaker(
        talea, talea_denominator, prolation_addenda,
        lefts, middles, rights, left_lengths, right_lengths)

    divisions = [(3, 16), (3, 8)]
    music = maker(divisions)

    music = sequencetools.flatten_sequence(music)
    staff = Staff(scoretools.make_spacer_skip_measures(divisions))
    scoretools.replace_contents_of_measures_in_expr(staff, music)

    r'''
    \new Staff {
        {
            \time 3/16
            \tweak #'text #tuplet-number::calc-fraction-text
            \times 3/5 {
                r4
                c'16
            }
        }
        {
            \time 3/8
            \tweak #'text #tuplet-number::calc-fraction-text
            \times 3/4 {
                c'8.
                c'4
                r16
            }
        }
    }
    '''

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 3/16
                \tweak #'text #tuplet-number::calc-fraction-text
                \times 3/5 {
                    r4
                    c'16
                }
            }
            {
                \time 3/8
                \tweak #'text #tuplet-number::calc-fraction-text
                \times 3/4 {
                    c'8.
                    c'4
                    r16
                }
            }
        }
        '''
        )


def test_rhythmmakertools_OutputBurnishedTaleaRhythmMaker___call___03():

    talea, talea_denominator, prolation_addenda = [1, 2, 3], 16, [0, 2]
    lefts, middles, rights = [-1], [0], [-1]
    left_lengths, right_lengths = [1], [1]
    secondary_divisions = [9]
    maker = rhythmmakertools.OutputBurnishedTaleaRhythmMaker(
        talea, talea_denominator, prolation_addenda,
        lefts, middles, rights, left_lengths, right_lengths, secondary_divisions)

    divisions = [(3, 8), (4, 8)]
    music = maker(divisions)

    music = sequencetools.flatten_sequence(music)
    staff = Staff(scoretools.make_spacer_skip_measures(divisions))
    scoretools.replace_contents_of_measures_in_expr(staff, music)

    r'''
    \new Staff {
        {
            \time 3/8
            {
                r16
                c'8
                c'8.
            }
        }
        {
            \time 4/8
            \tweak #'text #tuplet-number::calc-fraction-text
            \times 3/5 {
                c'16
                c'8
                c'8
            }
            {
                c'16
                c'16
                c'8
                r16
            }
        }
    }
    '''

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 3/8
                {
                    r16
                    c'8
                    c'8.
                }
            }
            {
                \time 4/8
                \tweak #'text #tuplet-number::calc-fraction-text
                \times 3/5 {
                    c'16
                    c'8
                    c'8
                }
                {
                    c'16
                    c'16
                    c'8
                    r16
                }
            }
        }
        '''
        )


def test_rhythmmakertools_OutputBurnishedTaleaRhythmMaker___call___04():

    talea, talea_denominator, prolation_addenda  = [1], 8, []
    lefts, middles, rights = [-1], [0], [-1]
    left_lengths, right_lengths = [1], [2]
    maker = rhythmmakertools.OutputBurnishedTaleaRhythmMaker(
        talea, talea_denominator, prolation_addenda,
        lefts, middles, rights,
        left_lengths, right_lengths)

    divisions = [(8, 8)]
    music = maker(divisions)

    music = sequencetools.flatten_sequence(music)
    staff = Staff(scoretools.make_spacer_skip_measures(divisions))
    scoretools.replace_contents_of_measures_in_expr(staff, music)

    r'''
    \new Staff {
        {
            \time 8/8
            r8
            c'8
            c'8
            c'8
            c'8
            c'8
            r8
            r8
        }
    }
    '''

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 8/8
                r8
                c'8
                c'8
                c'8
                c'8
                c'8
                r8
                r8
            }
        }
        '''
        )



# -*- encoding: utf-8 -*-
import abc
import re
from abjad.tools.abctools.AbjadObject import AbjadObject
from abjad.tools.pitchtools.PitchClass import PitchClass
from abjad.tools.pitchtools.Accidental import Accidental
from abjad.tools.pitchtools.Octave import Octave


class Pitch(AbjadObject):
    '''Pitch base class.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    _diatonic_pitch_name_regex_body = '''
        {}  # exactly one diatonic pitch-class name
        {}  # followed by exactly one octave tick string
        '''.format(
        PitchClass._diatonic_pitch_class_name_regex_body,
        Octave._octave_tick_regex_body,
        )

    _diatonic_pitch_name_regex = re.compile(
        '^{}$'.format(_diatonic_pitch_name_regex_body),
        re.VERBOSE,
        )

    _pitch_class_octave_number_regex_body = '''
        ([A-G])         # exactly one diatonic pitch-class letter
        {}                # plus an optional symbolic accidental string
        ([-]?           # plus an optional negative sign
        [0-9]+)         # plus one or more digits
        '''.format(
        Accidental._symbolic_string_regex_body,
        )

    _pitch_class_octave_number_regex = re.compile(
        '^{}$'.format(_pitch_class_octave_number_regex_body),
        re.VERBOSE,
        )

    _pitch_name_regex_body = '''
        {}  # exactly one diatonic pitch-class name
        {}  # followed by exactly one alphabetic accidental name
        {}  # followed by exactly one octave tick string
        '''.format(
        PitchClass._diatonic_pitch_class_name_regex_body,
        Accidental._alphabetic_accidental_regex_body,
        Octave._octave_tick_regex_body,
        )

    _pitch_name_regex = re.compile(
        '^{}$'.format(_pitch_name_regex_body),
        re.VERBOSE,
        )

    ### INITIALIZER ###

    @abc.abstractmethod
    def __init__(self):
        pass

    ### SPECIAL METHODS ###

    @abc.abstractmethod
    def __abs__(self):
        r'''Absolute value of pitch.
        '''
        message = 'TODO: all pitch-related classes must implement abs.'
        raise NotImplementedError(message)

    @abc.abstractmethod
    def __float__(self):
        r'''Changes pitch to float.

        Returns float.
        '''
        message = 'TODO: all pitch-related classes must implement float.'
        raise NotImplementedError(message)

    def __format__(self, format_specification=''):
        r'''Formats pitch.

        Set `format_specification` to `''`, `'lilypond'` or `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'lilypond'):
            return self._lilypond_format
        elif format_specification == 'storage':
            return systemtools.StorageFormatManager.get_storage_format(self)
        return str(self)

    def __hash__(self):
        r'''Hashes pitch.

        Returns integer.
        '''
        return hash(repr(self))

    def __illustrate__(self):
        r'''Illustrates pitch.

        Returns LilyPond file.
        '''
        from abjad.tools import durationtools
        from abjad.tools import lilypondfiletools
        from abjad.tools import markuptools
        from abjad.tools import pitchtools
        from abjad.tools import scoretools
        from abjad.tools.topleveltools import attach
        from abjad.tools.topleveltools import override
        pitch = pitchtools.NamedPitch(self)
        note = scoretools.Note(pitch, 1)
        attach(durationtools.Multiplier(1, 4), note)
        clef = pitchtools.suggest_clef_for_named_pitches([pitch])
        staff = scoretools.Staff()
        attach(clef, staff)
        staff.append(note)
        override(staff).time_signature.stencil = False
        lilypond_file = lilypondfiletools.make_basic_lilypond_file(staff)
        lilypond_file.header_block.tagline = markuptools.Markup('""')
        return lilypond_file

    @abc.abstractmethod
    def __int__(self):
        r'''Changes pitch to integer.

        Returns integer.
        '''
        message = 'TODO: all pitch-related classes must implement int.'
        raise NotImplementedError(message)

    ### PRIVATE PROPERTIES ###

    @abc.abstractproperty
    def _lilypond_format(self):
        r'''LilyPond input format.
        '''
        raise NotImplementedError

    ### PUBLIC METHODS ###

    @abc.abstractmethod
    def apply_accidental(self, accidental=None):
        r'''Applies `accidental` to pitch.

        Returns new pitch.
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def invert(self, axis=None):
        r'''Inverts pitch about `axis`.

        Returns new pitch.
        '''
        raise NotImplementedError

    @staticmethod
    def is_diatonic_pitch_name(expr):
        '''True when `expr` is a diatonic pitch name. Otherwise false.

        ::

            >>> pitchtools.Pitch.is_diatonic_pitch_name("c''")
            True

        The regex ``(^[a-g,A-G])(,+|'+|)$`` underlies this predicate.

        Returns boolean.
        '''
        if not isinstance(expr, str):
            return False
        return bool(Pitch._diatonic_pitch_name_regex.match(expr))

    @staticmethod
    def is_diatonic_pitch_number(expr):
        '''True when `expr` is a diatonic pitch number. Otherwise false.

        ::

            >>> pitchtools.Pitch.is_diatonic_pitch_number(7)
            True

        The diatonic pitch numbers are equal to the set of integers.

        Returns boolean.
        '''
        return isinstance(expr, (int, long))

    @staticmethod
    def is_pitch_carrier(expr):
        '''True when `expr` is an Abjad pitch, note, note-head of chord
        instance. Otherwise false.

        ::

            >>> note = Note("c'4")
            >>> pitchtools.Pitch.is_pitch_carrier(note)
            True

        Returns boolean.
        '''
        from abjad.tools import scoretools
        from abjad.tools import scoretools
        from abjad.tools import pitchtools
        return isinstance(
            expr, (
                pitchtools.NamedPitch,
                scoretools.Note,
                scoretools.NoteHead,
                scoretools.Chord
                )
            )

    @staticmethod
    def is_pitch_class_octave_number_string(expr):
        '''True when `expr` is a pitch-class / octave number string. Otherwise 
        false:

        ::

            >>> pitchtools.Pitch.is_pitch_class_octave_number_string('C#2')
            True

        Quartertone accidentals are supported.

        The regex ``^([A-G])([#]{1,2}|[b]{1,2}|[#]?[+]|[b]?[~]|)([-]?[0-9]+)$``
        underlies this predicate.

        Returns boolean.
        '''
        if not isinstance(expr, str):
            return False
        return bool(Pitch._pitch_class_octave_number_regex.match(expr))

    @staticmethod
    def is_pitch_name(expr):
        '''True `expr` is a pitch name. Otherwise false.

        ::

            >>> pitchtools.Pitch.is_pitch_name('c,')
            True

        The regex ``^([a-g,A-G])(([s]{1,2}|[f]{1,2}|t?q?[f,s]|)!?)(,+|'+|)$``
        underlies this predicate.

        Returns boolean.
        '''
        if not isinstance(expr, str):
            return False
        return bool(Pitch._pitch_name_regex.match(expr))

    @staticmethod
    def is_pitch_number(expr):
        '''True when `expr` is a pitch number. Otherwise false.

        ::

            >>> pitchtools.Pitch.is_pitch_number(13)
            True

        The pitch numbers are equal to the set of all integers in
        union with the set of all integers plus of minus ``0.5``.

        Returns boolean.
        '''
        if isinstance(expr, (int, long, float)):
            return expr % 0.5 == 0
        return False

    @abc.abstractmethod
    def multiply(self, n=1):
        r'''Multiplies pitch by `n`.
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def transpose(self, expr):
        r'''Transposes pitch by `expr`.

        Returns new pitch.
        '''
        raise NotImplementedError

    ### PUBLIC PROPERTIES ###

    @abc.abstractproperty
    def accidental(self):
        r'''Accidental of pitch.
        '''
        raise NotImplementedError

    @property
    def accidental_spelling(self):
        r'''Accidental spelling of Abjad session.

        ::

            >>> NamedPitch("c").accidental_spelling
            'mixed'

        Returns string.
        '''
        from abjad import abjad_configuration
        return abjad_configuration['accidental_spelling']

    @abc.abstractproperty
    def alteration_in_semitones(self):
        r'''Alteration of pitch in semitones.
        '''
        raise NotImplementedError

    @abc.abstractproperty
    def diatonic_pitch_class_name(self):
        r'''Diatonic pitch-class name of pitch.
        '''
        raise NotImplementedError

    @abc.abstractproperty
    def diatonic_pitch_class_number(self):
        r'''Diatonic pitch-class number of pitch.
        '''
        raise NotImplementedError

    @abc.abstractproperty
    def diatonic_pitch_name(self):
        r'''Diatonic pitch name of pitch.
        '''
        raise NotImplementedError

    @abc.abstractproperty
    def diatonic_pitch_number(self):
        r'''Diatonic pitch number of pitch.
        '''
        raise NotImplementedError

    @abc.abstractproperty
    def named_pitch(self):
        r'''Named pitch corresponding to pitch.
        '''
        raise NotImplementedError

    @abc.abstractproperty
    def named_pitch_class(self):
        r'''Named pitch-class corresponding to pitch.
        '''
        raise NotImplementedError

    @abc.abstractproperty
    def numbered_pitch(self):
        r'''Numbered pitch corresponding to pitch.
        '''
        raise NotImplementedError

    @abc.abstractproperty
    def numbered_pitch_class(self):
        r'''Numbered pitch-class corresponding to pitch.
        '''
        raise NotImplementedError

    @abc.abstractproperty
    def octave(self):
        r'''Octave of pitch.

        Returns octave.
        '''
        raise NotImplementedError

    @abc.abstractproperty
    def octave_number(self):
        r'''Octave number of pitch.
        
        Returns integer.
        '''
        raise NotImplementedError

    @abc.abstractproperty
    def pitch_class_name(self):
        r'''Pitch-class name corresponding to pitch.
        
        Returns string.
        '''
        raise NotImplementedError

    @abc.abstractproperty
    def pitch_class_number(self):
        r'''Pitch-class number of pitch.
        
        Returns number
        '''
        raise NotImplementedError

    @abc.abstractproperty
    def pitch_class_octave_label(self):
        r'''Pitch-class / octave label of pitch.
        '''
        raise NotImplementedError

    @abc.abstractproperty
    def pitch_name(self):
        r'''Pitch name of pitch.
        
        Returns string.
        '''
        raise NotImplementedError

    @abc.abstractproperty
    def pitch_number(self):
        r'''Pitch number of pitch.
        
        Returns number.
        '''
        raise NotImplementedError



# -*- encoding: utf-8 -*-
from abjad.tools import sequencetools
from abjad.tools.pitchtools.Set import Set


class IntervalSet(Set):
    r'''An interval set.
    '''

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### PRIVATE PROPERTIES ###

    @property
    def _named_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.NamedInterval
    
    @property
    def _numbered_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.NumberedInterval

    @property
    def _parent_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.Interval

    ### PUBLIC METHODS ###

    @classmethod
    def from_selection(cls, selection, item_class=None, custom_identifier=None):
        r'''Initialize interval set from component selection:

        ::

            >>> staff_1 = Staff("c'4 <d' fs' a'>4 b2")
            >>> staff_2 = Staff("c4. r8 g2")
            >>> selection = select((staff_1, staff_2))
            >>> intervals = pitchtools.IntervalSet.from_selection(
            ...     selection)
            >>> for interval in intervals:
            ...     interval
            ...
            NamedInterval('+m3')
            NamedInterval('+M3')
            NamedInterval('+P4')
            NamedInterval('+M7')
            NamedInterval('-M6')
            NamedInterval('+m2')
            NamedInterval('+aug11')
            NamedInterval('-P5')
            NamedInterval('+M13')
            NamedInterval('+P8')
            NamedInterval('-M3')
            NamedInterval('-aug4')
            NamedInterval('+M9')
            NamedInterval('+m7')
            NamedInterval('-M2')
            NamedInterval('-m3')
            NamedInterval('+P5')
        
        Returns interval set.
        '''
        from abjad.tools import pitchtools
        pitch_segment = pitchtools.PitchSegment.from_selection(selection)
        pairs = sequencetools.yield_all_unordered_pairs_of_sequence(
            pitch_segment)
        intervals = (second - first for first, second in pairs)
        return cls(
            tokens=intervals,
            item_class=item_class,
            custom_identifier=custom_identifier,
            )



# -*- encoding: utf-8 -*-
import copy
from abjad.tools.abctools.AbjadObject import AbjadObject


class Annotation(AbjadObject):
    r'''An annotation.

    ::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> pitch = NamedPitch('ds')
        >>> annotation = indicatortools.Annotation('special pitch', pitch)
        >>> attach(annotation, staff[0])
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> print format(staff)
        \new Staff {
            c'8
            d'8
            e'8
            f'8
        }

    Annotations contribute no formatting.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_name', 
        '_value',
        )

    ### INITIALIZER ###

    def __init__(self, *args):
        if len(args) == 1 and isinstance(args[0], type(self)):
            self._name = copy.copy(args[0].name)
            self._value = copy.copy(args[0].value)
        elif len(args) == 1 and not isinstance(args[0], type(self)):
            self._name = copy.copy(args[0])
            self._value = None
        elif len(args) == 2:
            self._name = copy.copy(args[0])
            self._value = copy.copy(args[1])
        elif len(args) == 0:
            name = 'annotation'
            value = None
        else:
            message = 'can not initialize {}: {!r}'
            message = message.format(type(self).__name__, args)
            raise ValueError(message)

    ### SPECIAL METHODS ###

    def __copy__(self, *args):
        r'''Copies annotation.

        Returns new annotation.
        '''
        return type(self)(self.name, self.value)

    def __eq__(self, arg):
        r'''True when arg is an annotation with name and value
        equal to those of this annotation. Otherwise false.

        Returns boolean.
        '''
        if isinstance(arg, type(self)):
            if self.name == arg.name:
                if self.value == self.value:
                    return True
        return False

    ### PRIVATE PROPERTIES ###

    @property
    def _repr_specification(self):
        return self._storage_format_specification.__makenew__(
            is_indented=False,
            )

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        return systemtools.StorageFormatSpecification(
            self,
            positional_argument_values=(
                self.name,
                self.value
                ),
            )

    ### PUBLIC PROPERTIES ###

    @property
    def name(self):
        r'''Name of annotation.

        ::

            >>> annotation.name
            'special pitch'

        Returns string.
        '''
        return self._name

    @property
    def value(self):
        r'''Value of annotation.

        ::

            >>> annotation.value
            NamedPitch('ds')

        Returns arbitrary object.
        '''
        return self._value



# -*- encoding: utf-8 -*-
import pickle
from abjad import *



def test_quantizationtools_QGrid_pickle_01():

    q_grid = quantizationtools.QGrid(
        root_node=quantizationtools.QGridContainer(
            preprolated_duration=1, children=[
            quantizationtools.QGridLeaf(
                preprolated_duration=1, q_event_proxies=[
                quantizationtools.QEventProxy(
                    quantizationtools.SilentQEvent(100),
                    0.5
                    )
                ])
            ]),
        next_downbeat=quantizationtools.QGridLeaf(
            preprolated_duration=1,
            q_event_proxies=[
                quantizationtools.QEventProxy(
                    quantizationtools.TerminalQEvent(200),
                    0.9
                    )
                ]
            )
        )

    pickled = pickle.loads(pickle.dumps(q_grid))

    assert pickled == q_grid
    assert pickled is not q_grid



# -*- encoding: utf-8 -*-
from abjad.tools.pitchtools.Segment import Segment
from abjad.tools.topleveltools import iterate
from abjad.tools.topleveltools import select


class PitchSegment(Segment):
    r'''A pitch segment.

    ::

        >>> numbered_pitch_segment = pitchtools.PitchSegment(
        ...     tokens=[-2, -1.5, 6, 7, -1.5, 7],
        ...     item_class=pitchtools.NumberedPitch,
        ...     )
        >>> numbered_pitch_segment
        PitchSegment([-2, -1.5, 6, 7, -1.5, 7])

    ::

        >>> show(numbered_pitch_segment) # doctest: +SKIP

    ::

        >>> named_pitch_segment = pitchtools.PitchSegment(
        ...     ['bf,', 'aqs', "fs'", "g'", 'bqf', "g'"],
        ...     item_class=NamedPitch,
        ...     )
        >>> named_pitch_segment
        PitchSegment(['bf,', 'aqs', "fs'", "g'", 'bqf', "g'"])

    ::

        >>> show(named_pitch_segment) # doctest: +SKIP

    '''

    ### CLASS VARIABLES ###

    _default_positional_input_arguments = (
        [-2, -1.5, 6, 7, -1.5, 7],
        )

    __slots__ = ()

    ### SPECIAL METHODS ###

    def __illustrate__(self):
        r'''Illustrates pitch segment.

        ::

            >>> named_pitch_segment = pitchtools.PitchSegment(
            ...     ['bf,', 'aqs', "fs'", "g'", 'bqf', "g'"],
            ...     item_class=NamedPitch,
            ...     )
            >>> show(named_pitch_segment) # doctest: +SKIP

        Returns LilyPond file.
        '''
        from abjad.tools import durationtools
        from abjad.tools import lilypondfiletools
        from abjad.tools import markuptools
        from abjad.tools import pitchtools
        from abjad.tools import scoretools
        from abjad.tools import spannertools
        from abjad.tools.topleveltools import attach
        from abjad.tools.topleveltools import iterate
        from abjad.tools.topleveltools import override
        named_pitches = [pitchtools.NamedPitch(x) for x in self]
        notes = scoretools.make_notes(named_pitches, [1])
        score, treble_staff, bass_staff = \
            scoretools.make_piano_sketch_score_from_leaves(notes)
        for leaf in iterate(score).by_class(scoretools.Leaf):
            attach(durationtools.Multiplier(1, 8), leaf)
        override(score).rest.transparent = True
        lilypond_file = lilypondfiletools.make_basic_lilypond_file(score)
        lilypond_file.header_block.tagline = markuptools.Markup('""')
        return lilypond_file

    ### PRIVATE PROPERTIES ###

    @property
    def _named_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.NamedPitch

    @property
    def _numbered_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.NumberedPitch

    @property
    def _parent_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.Pitch

    ### PUBLIC METHODS ###

    @classmethod
    def from_selection(
        cls, 
        selection, 
        item_class=None, 
        custom_identifier=None,
        ):
        r'''Makes pitch segment from `selection`.

        ::

            >>> staff_1 = Staff("c'4 <d' fs' a'>4 b2")
            >>> staff_2 = Staff("c4. r8 g2")
            >>> selection = select((staff_1, staff_2))
            >>> pitch_segment = pitchtools.PitchSegment.from_selection(
            ...     selection)
            >>> pitch_segment
            PitchSegment(["c'", "d'", "fs'", "a'", 'b', 'c', 'g'])

        ::

            >>> show(pitch_segment) # doctest: +SKIP

        Returns pitch segment.
        '''
        from abjad.tools import pitchtools
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        if not isinstance(selection, selectiontools.Selection):
            selection = select(selection)
        named_pitches = []
        for component in iterate(selection).by_class(
            (scoretools.Note, scoretools.Chord)):
            if hasattr(component, 'written_pitches'):
                named_pitches.extend(component.written_pitches)
            elif hasattr(component, 'written_pitch'):
                named_pitches.append(component.written_pitch)
        return cls(
            tokens=named_pitches,
            item_class=item_class,
            custom_identifier=custom_identifier,
            )

    def invert(self, axis):
        r'''Inverts pitch segment about `axis`.

        Returns new pitch segment.
        '''
        tokens = (pitch.invert(axis) for pitch in self)
        return self.__makenew__(tokens=tokens)

    def is_equivalent_under_transposition(self, expr):
        r'''True if pitch segment is equivalent to `expr` under transposition.
        Otherwise false.

        Returns boolean.
        '''
        from abjad.tools import pitchtools
        if not isinstance(expr, type(self)):
            return False
        if not len(self) == len(expr):
            return False
        difference = -(pitchtools.NamedPitch(expr[0], 4) -
            pitchtools.NamedPitch(self[0], 4))
        new_pitches = (x + difference for x in self)
        new_pitches = self.__makenew__(tokens=new_pitch)
        return expr == new_pitches

    def make_notes(self, n=None, written_duration=None):
        r'''Makes first `n` notes in pitch segment.

        Set `n` equal to `n` or length of segment.

        Set `written_duration` equal to `written_duration` or ``1/8``:

        ::

            >>> notes = named_pitch_segment.make_notes()
            >>> staff = Staff(notes)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print format(staff)
            \new Staff {
                bf,8
                aqs8
                fs'8
                g'8
                bqf8
                g'8
            }

        Allows nonassignable `written_duration`:

        ::

            >>> notes = named_pitch_segment.make_notes(4, Duration(5, 16))
            >>> staff = Staff(notes)
            >>> time_signature = TimeSignature((5, 4))
            >>> attach(time_signature, staff)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print format(staff)
            \new Staff {
                \time 5/4
                bf,4 ~
                bf,16
                aqs4 ~
                aqs16
                fs'4 ~
                fs'16
                g'4 ~
                g'16
            }

        Returns list of notes.
        '''
        from abjad.tools import durationtools
        from abjad.tools import scoretools
        n = n or len(self)
        written_duration = written_duration or durationtools.Duration(1, 8)
        result = scoretools.make_notes([0] * n, [written_duration])
        for i, logical_tie in enumerate(iterate(result).by_logical_tie()):
            pitch = self[i % len(self)]
            for note in logical_tie:
                note.written_pitch = pitch
        return result

    def retrograde(self):
        r'''Retrograde of pitch segment.

        ::

            >>> result = named_pitch_segment.retrograde()
            >>> result
            PitchSegment(["g'", 'bqf', "g'", "fs'", 'aqs', 'bf,'])

        ::

            >>> show(result) # doctest: +SKIP

        Returns new pitch segment.
        '''
        return self.__makenew__(tokens=reversed(self))

    def rotate(self, n):
        r'''Rotates pitch segment.

        ::

            >>> result = numbered_pitch_segment.rotate(1)
            >>> result
            PitchSegment([7, -2, -1.5, 6, 7, -1.5])

        ::

            >>> show(result) # doctest: +SKIP
        
        ::

            >>> result = named_pitch_segment.rotate(-2)
            >>> result
            PitchSegment(["fs'", "g'", 'bqf', "g'", 'bf,', 'aqs'])

        ::

            >>> show(result) # doctest: +SKIP
        
        Returns new pitch segment.
        '''
        from abjad.tools import sequencetools
        tokens = sequencetools.rotate_sequence(self._collection, n)
        return self.__makenew__(tokens=tokens)

    def transpose(self, expr):
        r'''Transposes pitch segment by `expr`.

        Returns new pitch segment.
        '''
        tokens = (pitch.transpose(expr) for pitch in self)
        return self.__makenew__(tokens=tokens)

    ### PUBLIC PROPERTIES ###

    @property
    def has_duplicates(self):
        r'''True if pitch segment has duplicate items. Otherwise false.

        ::

            >>> pitch_class_segment = pitchtools.PitchSegment(
            ...     tokens=[-2, -1.5, 6, 7, -1.5, 7],
            ...     )
            >>> pitch_class_segment.has_duplicates
            True

        ::

            >>> pitch_class_segment = pitchtools.PitchSegment(
            ...     tokens="c d e f g a b",
            ...     )
            >>> pitch_class_segment.has_duplicates
            False

        Returns boolean.
        '''
        from abjad.tools import pitchtools
        return len(pitchtools.PitchSet(self)) < len(self)

    @property
    def inflection_point_count(self):
        r'''Inflection point count of pitch segment.

        Returns nonnegative integer.
        '''
        return len(self.local_minima) + len(self.local_maxima)

    @property
    def local_maxima(self):
        r'''Local maxima of pitch segment.

        Returns tuple.
        '''
        result = []
        if 3 <= len(self):
            for i in range(1, len(self) - 1):
                left, middle, right = self[i - 1], self[i], self[i + 1]
                if left < middle and right < middle:
                    result.append(middle)
        return tuple(result)

    @property
    def local_minima(self):
        r'''Local minima of pitch segment.

        Returns tuple.
        '''
        result = []
        if 3 <= len(self):
            for i in range(1, len(self) - 1):
                left, middle, right = self[i - 1], self[i], self[i + 1]
                if middle < left and middle < right:
                    result.append(middle)
        return tuple(result)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import setuptools
import os
version_file_path = os.path.join(
    os.path.dirname(__file__),
    'abjad',
    '_version.py'
    )
execfile(version_file_path, {}, locals())

description = 'Abjad is a Python API for Formalized Score Control.'

long_description = 'Abjad is an interactive software system designed'
long_description += ' to help composers build up complex pieces of'
long_description += ' music notation in an iterative and incremental way.'
long_description += ' Use Abjad to create a symbolic representation of all'
long_description += ' the notes, rests, staves, tuplets, beams and slurs'
long_description += ' in any score.'
long_description += ' Because Abjad extends the Python programming language,'
long_description += ' you can use Abjad to make systematic changes to'
long_description += ' your music as you work.'
long_description += ' And because Abjad wraps the powerful LilyPond music'
long_description += ' notation package, you can use Abjad to control'
long_description += ' the typographic details of all the symbols on the page.'

author = [
    'Trevor Bača',
    'Josiah Wolf Oberholtzer',
    'Víctor Adán',
    ]
author = ', '.join(author)

author_email = [
    'trevorbaca@gmail.com',
    'josiah.oberholtzer@gmail.com',
    'contact@victoradan.net',
    ]
author_email = ', '.join(author_email)

keywords = [
    'music composition',
    'music notation',
    'formalized score control',
    'lilypond',
    ]
keywords = ', '.join(keywords)

setuptools.setup(
    author=author,
    author_email=author_email,
    description=description,
    entry_points={
        'console_scripts': [
            'abjad = abjad.tools.systemtools.run_abjad:run_abjad',
            'ajv = abjad.tools.developerscripttools.run_ajv:run_ajv',
            ]
        },
    include_package_data=True,
    install_requires=[
        'pytest >= 2.1',
        'Sphinx >= 1.0.7',
        'ply >= 3.4',
        'configobj >= 4.7.2',
        ],
    keywords=keywords,
    license='GPL',
    long_description=long_description,
    name='Abjad',
    packages=setuptools.find_packages(exclude='experimental'),
    platforms='Any',
    url='http://www.projectabjad.org',
    version=__version__,
    )



# -*- encoding: utf-8 -*-
from abjad import *


def test_pitcharraytools_PitchArrayCell_matches_cell_01():

    array = pitcharraytools.PitchArray([[1, 2, 1], [2, 1, 1]])
    array[0].cells[0].pitches.append(NamedPitch(0))
    array[0].cells[1].pitches.extend([NamedPitch(2), NamedPitch(4)])

    '''
    [c'] [d' e'     ] []
    [             ] [] []
    '''

    assert array[0].cells[0].matches_cell(array[0].cells[0])
    assert not array[0].cells[0].matches_cell(array[0].cells[1])
    assert not array[0].cells[0].matches_cell(array[0].cells[2])
    assert not array[0].cells[0].matches_cell(array[1].cells[0])
    assert not array[0].cells[0].matches_cell(array[1].cells[1])
    assert not array[0].cells[0].matches_cell(array[1].cells[2])

    assert not array[0].cells[2].matches_cell(array[0].cells[0])
    assert not array[0].cells[2].matches_cell(array[0].cells[1])
    assert array[0].cells[2].matches_cell(array[0].cells[2])
    assert not array[0].cells[2].matches_cell(array[1].cells[0])
    assert array[0].cells[2].matches_cell(array[1].cells[1])
    assert array[0].cells[2].matches_cell(array[1].cells[2])



# -*- encoding: utf-8 -*-
from abjad import *


def test_scoretools_NoteHeadInventory_append_01():
    r'''Append tweaked note head to chord.
    '''

    chord = Chord("<c' d'>4")
    note_head = scoretools.NoteHead("b'")
    note_head.tweak.style = 'harmonic'
    chord.note_heads.append(note_head)

    assert systemtools.TestManager.compare(
        chord,
        r'''
        <
            c'
            d'
            \tweak #'style #'harmonic
            b'
        >4
        '''
        )
        
    assert note_head._client is chord



# -*- encoding: utf-8 -*-
from abjad import *
from abjad.tools import sequencetools
from abjad.tools import rhythmmakertools


def test_rhythmmakertools_TaleaRhythmMaker___call___01():

    talea, talea_denominator, prolation_addenda = [-1, 4, -2, 3], 16, [3, 4]
    maker = rhythmmakertools.TaleaRhythmMaker(talea, talea_denominator, prolation_addenda)

    divisions = [(2, 8), (5, 8)]
    music = maker(divisions)

    music = sequencetools.flatten_sequence(music)
    staff = Staff(scoretools.make_spacer_skip_measures(divisions))
    scoretools.replace_contents_of_measures_in_expr(staff, music)

    r'''
    \new Staff {
        {
            \time 2/8
            \times 4/7 {
                r16
                c'4
                r8
            }
        }
        {
            \time 5/8
            \tweak #'text #tuplet-number::calc-fraction-text
            \times 5/7 {
                c'8.
                r16
                c'4
                r8
                c'8.
                r16
            }
        }
    }
    '''

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 2/8
                \times 4/7 {
                    r16
                    c'4
                    r8
                }
            }
            {
                \time 5/8
                \tweak #'text #tuplet-number::calc-fraction-text
                \times 5/7 {
                    c'8.
                    r16
                    c'4
                    r8
                    c'8.
                    r16
                }
            }
        }
        '''
        )


def test_rhythmmakertools_TaleaRhythmMaker___call___02():

    talea, talea_denominator, prolation_addenda = [-1, 4, -2, 3], 16, [3, 4]
    secondary_divisions = [6]
    maker = rhythmmakertools.TaleaRhythmMaker(talea, talea_denominator, prolation_addenda, secondary_divisions)

    divisions = [(2, 8), (5, 8)]
    music = maker(divisions)

    music = sequencetools.flatten_sequence(music)
    staff = Staff(scoretools.make_spacer_skip_measures(divisions))
    scoretools.replace_contents_of_measures_in_expr(staff, music)

    r'''
    \new Staff {
        {
            \time 2/8
            \times 4/7 {
                r16
                c'4
                r8
            }
        }
        {
            \time 5/8
            {
                c'8
            }
            \times 2/3 {
                c'16
                r16
                c'4
                r8
                c'16
            }
            {
                c'8
            }
        }
    }
    '''

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 2/8
                \times 4/7 {
                    r16
                    c'4
                    r8
                }
            }
            {
                \time 5/8
                {
                    c'8
                }
                \times 2/3 {
                    c'16
                    r16
                    c'4
                    r8
                    c'16
                }
                {
                    c'8
                }
            }
        }
        '''
        )



# -*- encoding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools import markuptools
from abjad.tools import pitchtools
from abjad.tools.instrumenttools.Instrument import Instrument


class UntunedPercussion(Instrument):
    r'''An untuned percussion instrument.

    ::

        >>> staff = Staff("c'4 d'4 e'4 fs'4")
        >>> untuned_percussion = instrumenttools.UntunedPercussion()
        >>> attach(untuned_percussion, staff)
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> print format(staff)
        \new Staff {
            \set Staff.instrumentName = \markup { Untuned percussion }
            \set Staff.shortInstrumentName = \markup { Perc. }
            c'4
            d'4
            e'4
            fs'4
        }

    '''

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        instrument_name='untuned percussion',
        short_instrument_name='perc.',
        instrument_name_markup=None,
        short_instrument_name_markup=None,
        allowable_clefs=('percussion',),
        pitch_range=None,
        sounding_pitch_of_written_middle_c=None,
        ):
        Instrument.__init__(
            self,
            instrument_name=instrument_name,
            short_instrument_name=short_instrument_name,
            instrument_name_markup=instrument_name_markup,
            short_instrument_name_markup=short_instrument_name_markup,
            allowable_clefs=allowable_clefs,
            pitch_range=pitch_range,
            sounding_pitch_of_written_middle_c=\
                sounding_pitch_of_written_middle_c,
            )
        self._performer_names.extend([
            'percussionist',
            ])

    ### CLASS VARIABLES ###

    known_untuned_percussion = list(sorted(set([
        'agogô',
        'anvil',
        'bass drum',
        'bongo drums',
        'cabasa',
        'cajón',
        'castanets',
        'caxixi',
        'claves',
        'conga drums',
        'cowbell',
        'cuíca',
        'djembe',
        'finger cymbals',
        'flexatone',
        'frame drum',
        'gong',
        'güiro',
        'jawbone',
        'maracas',
        'ratchet',
        'rattle',
        'sand blocks',
        'siren',
        'slapstick',
        'slide whistle',
        'snare drum',
        'steel drums',
        'tambourine',
        'temple blocks',
        'thunder machine',
        'thundersheet',
        'toms',
        'triangle',
        'vibraslap',
        'whistle',
        'wind chime',
        'wind machine',
        'wood block',
        ])))
        
    ### PUBLIC PROPERTIES ###

    @property
    def allowable_clefs(self):
        r'''Gets untuned percussion's allowable clefs.

        ..  container:: example

            ::

                >>> untuned_percussion.allowable_clefs
                ClefInventory([Clef('percussion')])

            ::

                >>> show(untuned_percussion.allowable_clefs) # doctest: +SKIP

        Returns clef inventory.
        '''
        return Instrument.allowable_clefs.fget(self)

    @property
    def instrument_name(self):
        r'''Gets untuned percussion's name.

        ..  container:: example

            ::

                >>> untuned_percussion.instrument_name
                'untuned percussion'

        Returns string.
        '''
        return Instrument.instrument_name.fget(self)

    @property
    def instrument_name_markup(self):
        r'''Gets untuned percussion's instrument name markup.

        ..  container:: example

            ::

                >>> untuned_percussion.instrument_name_markup
                Markup(('Untuned percussion',))

            ::

                >>> show(untuned_percussion.instrument_name_markup) # doctest: +SKIP

        Returns markup.
        '''
        return Instrument.instrument_name_markup.fget(self)

    @property
    def pitch_range(self):
        r'''Gets untuned percussion's range.

        ..  container:: example

            ::

                >>> untuned_percussion.pitch_range
                PitchRange('[A0, C8]')

            ::

                >>> show(untuned_percussion.pitch_range) # doctest: +SKIP

        Returns pitch range.
        '''
        return Instrument.pitch_range.fget(self)

    @property
    def short_instrument_name(self):
        r'''Gets untuned_percussion's short instrument name.

        ..  container:: example

            ::

                >>> untuned_percussion.short_instrument_name
                'perc.'

        Returns string.
        '''
        return Instrument.short_instrument_name.fget(self)

    @property
    def short_instrument_name_markup(self):
        r'''Gets untuned percussion's short instrument name markup.

        ..  container:: example

            ::

                >>> untuned_percussion.short_instrument_name_markup
                Markup(('Perc.',))

            ::

                >>> show(untuned_percussion.short_instrument_name_markup) # doctest: +SKIP

        Returns markup.
        '''
        return Instrument.short_instrument_name_markup.fget(self)

    @property
    def sounding_pitch_of_written_middle_c(self):
        r'''Gets sounding pitch of untuned percussion's written middle C.

        ..  container:: example

            ::

                >>> untuned_percussion.sounding_pitch_of_written_middle_c
                NamedPitch("c'")

            ::

                >>> show(untuned_percussion.sounding_pitch_of_written_middle_c) # doctest: +SKIP

        Returns named pitch.
        '''
        return Instrument.sounding_pitch_of_written_middle_c.fget(self)



# -*- encoding: utf-8 -*-


def remove_sequence_elements_at_indices(sequence, indices):
    '''Remove `sequence` elements at `indices`:

    ::

        >>> sequencetools.remove_sequence_elements_at_indices(range(20), [1, 16, 17, 18])
        [0, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 19]

    Ignore negative indices.

    Returns list.
    '''

    result = []

    for i, element in enumerate(sequence):
        if i not in indices:
            result.append(element)

    return result



# -*- encoding: utf-8 -*-
import math
from abjad.tools import mathtools


def split_sequence_extended_to_weights(sequence, weights, overhang=True):
    '''Split sequence extended to weights.

    ..  container:: example

        **Example 1.** Split sequence extended to weights with overhang:

        ::

            >>> sequencetools.split_sequence_extended_to_weights(
            ...     [1, 2, 3, 4, 5], [7, 7, 7], overhang=True)
            [[1, 2, 3, 1], [3, 4], [1, 1, 2, 3], [4, 5]]

    ..  container:: example

        **Example 2.** Split sequence extended to weights without overhang:

        ::

            >>> sequencetools.split_sequence_extended_to_weights(
            ...     [1, 2, 3, 4, 5], [7, 7, 7], overhang=False)
            [[1, 2, 3, 1], [3, 4], [1, 1, 2, 3]]

    Returns sequence of sequence objects.
    '''
    from abjad.tools import sequencetools

    n = int(math.ceil(float(mathtools.weight(weights)) / mathtools.weight(sequence)))

    sequence = sequencetools.repeat_sequence_n_times(sequence, n)

    return sequencetools.split_sequence_by_weights(sequence, weights, cyclic=False, overhang=overhang)



# -*- encoding: utf-8 -*-
import types
from abjad.tools.topleveltools import iterate


class Selection(object):
    r'''A selection of components.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_music',
        )

    ### INITIALIZER ###

    def __init__(self, music=None):
        music = self._coerce_music(music)
        self._music = tuple(music)

    ### SPECIAL METHODS ###

    def __add__(self, expr):
        r'''Cocatenates `expr` to selection.

        Returns new selection.
        '''
        assert isinstance(expr, (Selection, list, tuple))
        if isinstance(expr, Selection):
            music = self._music + expr._music
            return type(self)(music)
        elif isinstance(expr, (tuple, list)):
            music = self._music + tuple(expr)
        return type(self)(music)

    def __contains__(self, expr):
        r'''True when `expr` is in selection. Otherwise false.

        Returns boolean.
        '''
        return expr in self._music

    def __eq__(self, expr):
        r'''True when selection and `expr` are of the same type
        and when music of selection equals music of `expr`.
        Otherwise false.

        Returns boolean.
        '''
        if isinstance(expr, type(self)):
            return self._music == expr._music
        elif isinstance(expr, (list, tuple)):
            return self._music == tuple(expr)

    def __format__(self, format_specification=''):
        r'''Formats duration.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatManager.get_storage_format(self)
        return str(self)

    def __getitem__(self, expr):
        r'''Gets item `expr` from selection.

        Returns component from selection.
        '''
        result = self._music.__getitem__(expr)
        if isinstance(result, tuple):
            selection = type(self)()
            selection._music = result[:]
            result = selection
        return result

    def __len__(self):
        r'''Number of components in selection.

        Returns nonnegative integer.
        '''
        return len(self._music)

    def __ne__(self, expr):
        r'''True when selection does not equal `expr`. Otherwise false.

        Returns boolean.
        '''
        return not self == expr

    def __radd__(self, expr):
        r'''Concatenates selection to `expr`.

        Returns newly created selection.
        '''
        assert isinstance(expr, (type(self), list, tuple))
        if isinstance(expr, type(self)):
            music = expr._music + self._music
            return Selection(music)
        # eventually remove this permissive branch 
        # and force the use of selections only
        elif isinstance(expr, (tuple, list)):
            music = tuple(expr) + self._music
        return Selection(music)

    def __repr__(self):
        r'''Gets interpreter representation of selection.

        Returns string.
        '''
        return '{}{!r}'.format(type(self).__name__, self._music)

    ### PRIVATE PROPERTIES ###

    @property
    def _preprolated_duration(self):
        return sum(component._preprolated_duration for component in self)

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        return systemtools.StorageFormatSpecification(
            self,
            keyword_argument_names=(),
            positional_argument_values=(
                self._music,
                ),
            )

    ### PRIVATE METHODS ###

    @staticmethod
    def _all_are_components_in_same_logical_voice(
        expr, prototype=None, allow_orphans=True):
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        allowable_types = (
            list,
            tuple,
            types.GeneratorType,
            selectiontools.Selection,
            )
        if not isinstance(expr, allowable_types):
            return False
        prototype = prototype or (scoretools.Component,)
        if not isinstance(prototype, tuple):
            prototype = (prototype, )
        assert isinstance(prototype, tuple)
        if len(expr) == 0:
            return True
        all_are_orphans_of_correct_type = True
        if allow_orphans:
            for component in expr:
                if not isinstance(component, prototype):
                    all_are_orphans_of_correct_type = False
                    break
                if not component._get_parentage().is_orphan:
                    all_are_orphans_of_correct_type = False
                    break
            if all_are_orphans_of_correct_type:
                return True
        first = expr[0]
        if not isinstance(first, prototype):
            return False
        orphan_components = True
        if not first._get_parentage().is_orphan:
            orphan_components = False
        same_logical_voice = True
        first_signature = first._get_parentage().logical_voice
        for component in expr[1:]:
            parentage = component._get_parentage()
            if not parentage.is_orphan:
                orphan_components = False
            if not allow_orphans and orphan_components:
                return False
            if parentage.logical_voice != first_signature:
                same_logical_voice = False
            if not allow_orphans and not same_logical_voice:
                return False
            if allow_orphans and not orphan_components and \
                not same_logical_voice:
                return False
        return True
    
    @staticmethod
    def _all_are_contiguous_components_in_same_logical_voice(
        expr, prototype=None, allow_orphans=True):
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        allowable_types = (
            list,
            tuple,
            types.GeneratorType,
            selectiontools.Selection,
            )
        if not isinstance(expr, allowable_types):
            return False
        prototype = prototype or (scoretools.Component,)
        if not isinstance(prototype, tuple):
            prototype = (prototype, )
        assert isinstance(prototype, tuple)
        if len(expr) == 0:
            return True
        all_are_orphans_of_correct_type = True
        if allow_orphans:
            for component in expr:
                if not isinstance(component, prototype):
                    all_are_orphans_of_correct_type = False
                    break
                if not component._get_parentage().is_orphan:
                    all_are_orphans_of_correct_type = False
                    break
            if all_are_orphans_of_correct_type:
                return True
        if not allow_orphans:
            if any(x._get_parentage().is_orphan for x in expr):
                return False
        first = expr[0]
        if not isinstance(first, prototype):
            return False
        first_parentage = first._get_parentage()
        first_logical_voice = first_parentage.logical_voice
        first_root = first_parentage.root
        previous = first
        for current in expr[1:]:
            current_parentage = current._get_parentage()
            current_logical_voice = \
                current_parentage.logical_voice
            # false if wrong type of component found
            if not isinstance(current, prototype):
                return False
            # false if in different logical voices
            if current_logical_voice != first_logical_voice:
                return False
            # false if components are in same score and are discontiguous
            if current_parentage.root == first_root:
                if not previous._is_immediate_temporal_successor_of(current):
                    return False
            previous = current
        return True

    @staticmethod
    def _all_are_contiguous_components_in_same_parent(
        expr, prototype=None, allow_orphans=True):
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        allowable_types = (
            list,
            tuple,
            types.GeneratorType,
            selectiontools.Selection,
            )
        if not isinstance(expr, allowable_types):
            return False
        prototype = prototype or (scoretools.Component, )
        if not isinstance(prototype, tuple):
            prototype = (prototype, )
        assert isinstance(prototype, tuple)
        if len(expr) == 0:
            return True
        all_are_orphans_of_correct_type = True
        if allow_orphans:
            for component in expr:
                if not isinstance(component, prototype):
                    all_are_orphans_of_correct_type = False
                    break
                if not component._get_parentage().is_orphan:
                    all_are_orphans_of_correct_type = False
                    break
            if all_are_orphans_of_correct_type:
                return True
        first = expr[0]
        if not isinstance(first, prototype):
            return False
        first_parent = first._parent
        if first_parent is None:
            if allow_orphans:
                orphan_components = True
            else:
                return False
        same_parent = True
        strictly_contiguous = True
        previous = first
        for current in expr[1:]:
            if not isinstance(current, prototype):
                return False
            if not current._get_parentage().is_orphan:
                orphan_components = False
            if not current._parent is first_parent:
                same_parent = False
            if not previous._is_immediate_temporal_successor_of(current):
                strictly_contiguous = False
            if (not allow_orphans or 
                (allow_orphans and not orphan_components)) and \
                (not same_parent or not strictly_contiguous):
                return False
            previous = current
        return True

    @staticmethod
    def _coerce_music(music):
        if music is None:
            music = ()
        elif isinstance(music, (tuple, list)):
            music = tuple(music)
        elif isinstance(music, Selection):
            music = tuple(music)
        elif isinstance(music, types.GeneratorType):
            music = tuple(music)
        else:
            music = (music, )
        return music

    def _get_component(self, prototype=None, n=0, recurse=True):
        from abjad.tools import scoretools
        prototype = prototype or (scoretools.Component,)
        if not isinstance(prototype, tuple):
            prototype = (prototype,)
        if 0 <= n:
            if recurse:
                components = iterate(self).by_class(prototype)
            else:
                components = self._music
            for i, x in enumerate(components):
                if i == n:
                    return x
        else:
            if recurse:
                components = iterate(self).by_class(
                    prototype, reverse=True)
            else:
                components = reversed(self._music)
            for i, x in enumerate(components):
                if i == abs(n) - 1:
                    return x

    def _get_spanner(self, prototype=None):
        spanners = self._get_spanners(prototype=prototype)
        if not spanners:
            message = 'no spanners found.'
            raise MissingSpannerError(message)
        elif len(spanners) == 1:
            return spanners.pop()
        else:
            message = 'multiple spanners found.'
            raise ExtraSpannerError(message)

    def _get_spanners(self, prototype=None):
        from abjad.tools import spannertools
        prototype = prototype or (spannertools.Spanner,)
        if not isinstance(prototype, tuple):
            prototype = (prototype, )
        assert isinstance(prototype, tuple)
        result = set()
        for component in self:
            spanners = component._get_spanners(prototype)
            result.update(spanners)
        return result

    def _iterate_components(self, recurse=True, reverse=False):
        if recurse:
            return iterate(self).by_class()
        else:
            return self._iterate_top_level_components(reverse=reverse)

    def _iterate_top_level_components(self, reverse=False):
        if reverse:
            for component in reversed(self):
                yield component
        else:
            for component in self:
                yield component

    def _set_parents(self, new_parent):
        r'''Not composer-safe.
        '''
        for component in self._music:
            component._set_parent(new_parent)

    ### PUBLIC METHODS ###

    def get_duration(self, in_seconds=False):
        r'''Gets duration of contiguous selection.

        Returns duration.
        '''
        return sum(
            component._get_duration(in_seconds=in_seconds) 
            for component in self
            )

    def get_spanners(self, prototype=None):
        r'''Gets spanners attached to any component in selection.

        Returns set.
        '''
        result = set()
        for component in self:
            spanners = component._get_spanners(prototype=prototype)
            result.update(spanners)
        return result



# -*- encoding: utf-8 -*-
import copy
import itertools
from abjad.tools import datastructuretools
from abjad.tools import durationtools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import iterate
from abjad.tools.topleveltools import mutate
from abjad.tools.selectiontools.Selection import Selection


class ContiguousSelection(Selection):
    r'''A time-contiguous selection of components.
    '''

    ### INITIALIZER ###

    def __init__(self, music=None):
        music = self._coerce_music(music)
        self._music = tuple(music)

    ### SPECIAL METHODS ###

    def __add__(self, expr):
        '''Adds `expr` to selection.

        Returns new selection.
        '''
        from abjad.tools import selectiontools
        assert isinstance(expr, (Selection, list, tuple))
        if isinstance(expr, type(self)):
            music = self._music + expr._music
        elif isinstance(expr, (tuple, list)):
            music = self._music + tuple(expr)
        if self._all_are_contiguous_components_in_same_logical_voice(music):
            return type(self)(music)
        else:
            return selectiontools.Selection(music)

    def __radd__(self, expr):
        '''Adds selection to `expr`.

        Returns new selection.
        '''
        assert isinstance(expr, (type(self), list, tuple))
        if isinstance(expr, type(self)):
            music = expr._music + self._music
            return type(self)(music)
        # eventually remove this permissive branch
        # and force the use of selections only
        elif isinstance(expr, (tuple, list)):
            music = tuple(expr) + self._music
        return ContiguousSelection(music)

    ### PRIVATE METHODS ###

    def _attach_tie_spanner_to_leaf_pair(self):
        from abjad.tools import scoretools
        from abjad.tools import spannertools
        assert len(self) == 2
        left_leaf, right_leaf = self
        assert isinstance(left_leaf, scoretools.Leaf), left_leaf
        assert isinstance(right_leaf, scoretools.Leaf), right_leaf
        left_logical_tie = left_leaf._get_logical_tie()
        right_logical_tie = right_leaf._get_logical_tie()
        prototype = (spannertools.Tie,)
        if left_logical_tie == right_logical_tie:
            return
        try:
            left_tie_spanner = left_leaf._get_spanner(prototype)
        except MissingSpannerError:
            left_tie_spanner = None
        try:
            right_tie_spanner = right_leaf._get_spanner(prototype)
        except MissingSpannerError:
            right_tie_spanner = None
        if left_tie_spanner is not None and right_tie_spanner is not None:
            left_tie_spanner.fuse(right_tie_spanner)
        elif left_tie_spanner is not None and right_tie_spanner is None:
            left_tie_spanner.append(right_leaf)
        elif left_tie_spanner is None and right_tie_spanner is not None:
            right_tie_spanner.append_left(left_leaf)
        elif left_tie_spanner is None and right_tie_spanner is None:
            tie = spannertools.Tie()
            attach(tie, [left_leaf, right_leaf])

    def _copy_and_include_enclosing_containers(self):
        from abjad.tools import scoretools
        # get governor
        parentage = self[0]._get_parentage(include_self=True)
        governor = parentage._get_governor()
        # find start and stop indices in governor
        governor_leaves = list(governor.select_leaves())
        for i, x in enumerate(governor_leaves):
            if x is self[0]:
                start_index_in_governor = i
        for i, x in enumerate(governor_leaves):
            if x is self[-1]:
                stop_index_in_governor = i
        # copy governor
        governor_copy = mutate(governor).copy()
        copied_leaves = governor_copy.select_leaves()
        # find start and stop leaves in copy of governor
        start_leaf = copied_leaves[start_index_in_governor]
        stop_leaf = copied_leaves[stop_index_in_governor]
        # trim governor copy forwards from first leaf
        found_start_leaf = False
        while not found_start_leaf:
            leaf = iterate(governor_copy).by_class(scoretools.Leaf).next()
            if leaf is start_leaf:
                found_start_leaf = True
            else:
                leaf._remove_and_shrink_durated_parent_containers()
        # trim governor copy backwards from last leaf
        found_stop_leaf = False
        while not found_stop_leaf:
            reverse_iterator = iterate(governor_copy).by_class(
                scoretools.Leaf, reverse=True)
            leaf = reverse_iterator.next()
            if leaf is stop_leaf:
                found_stop_leaf = True
            else:
                leaf._remove_and_shrink_durated_parent_containers()
        # return trimmed governor copy
        return governor_copy

    def _fuse(self):
        from abjad.tools import scoretools
        assert self._all_are_contiguous_components_in_same_logical_voice(self)
        if all(isinstance(x, scoretools.Leaf) for x in self):
            return self._fuse_leaves()
        elif all(isinstance(x, scoretools.Tuplet) for x in self):
            return self._fuse_tuplets()
        elif all(isinstance(x, scoretools.Measure) for x in self):
            return self._fuse_measures()
        else:
            message = 'can not fuse.'
            raise Exception(message)

    def _fuse_leaves(self):
        from abjad.tools import scoretools
        assert self._all_are_contiguous_components_in_same_logical_voice(self)
        assert all(isinstance(x, scoretools.Leaf) for x in self)
        leaves = self
        if len(leaves) <= 1:
            return leaves
        total_preprolated = leaves._preprolated_duration
        for leaf in leaves[1:]:
            parent = leaf._parent
            if parent:
                index = parent.index(leaf)
                del(parent[index])
        return leaves[0]._set_duration(total_preprolated)

    def _fuse_measures(self):
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        # check input
        assert self._all_are_contiguous_components_in_same_parent(
            self, prototype=(scoretools.Measure, ))
        # return none on empty measures
        if len(self) == 0:
            return None
        # TODO: instantiate a new measure
        #       instead of returning a reference to existing measure
        if len(self) == 1:
            return self[0]
        selection = selectiontools.SliceSelection(self)
        parent, start, stop = selection._get_parent_and_start_stop_indices()
        old_denominators = []
        new_duration = durationtools.Duration(0)
        for measure in self:
            effective_time_signature = measure.time_signature
            old_denominators.append(effective_time_signature.denominator)
            new_duration += effective_time_signature.duration
        new_time_signature = \
            measure._duration_and_possible_denominators_to_time_signature(
            new_duration,
            old_denominators,
            )
        music = []
        for measure in self:
            # scale before reassignment to prevent logical tie scale drama
            signature = measure.time_signature
            prolation = signature.implied_prolation
            multiplier = prolation / new_time_signature.implied_prolation
            measure._scale_contents(multiplier)
            measure_music = measure[:]
            measure_music._set_parents(None)
            music += measure_music
        new_measure = scoretools.Measure(new_time_signature, music)
        if parent is not None:
            self._give_dominant_spanners([new_measure])
        self._set_parents(None)
        if parent is not None:
            parent.insert(start, new_measure)
        return new_measure

    def _fuse_tuplets(self):
        from abjad.tools import scoretools
        assert self._all_are_contiguous_components_in_same_parent(
            self, prototype=(scoretools.Tuplet,))
        if len(self) == 0:
            return None
        first = self[0]
        first_multiplier = first.multiplier
        first_type = type(first)
        for tuplet in self[1:]:
            if tuplet.multiplier != first_multiplier:
                message = 'tuplets must carry same multiplier.'
                raise ValueError(message)
            if type(tuplet) != first_type:
                message = 'tuplets must be same type.'
                raise TypeError(message)
        if isinstance(first, scoretools.FixedDurationTuplet):
            total_contents_duration = sum(
                [x._contents_duration for x in self])
            new_target_duration = first_multiplier * total_contents_duration
            new_tuplet = scoretools.FixedDurationTuplet(
                new_target_duration, [])
        elif isinstance(first, scoretools.Tuplet):
            new_tuplet = scoretools.Tuplet(first_multiplier, [])
        else:
            message = 'unknown tuplet type.'
            raise TypeError(message)
        wrapped = False
        if self[0]._get_parentage().root is not \
            self[-1]._get_parentage().root:
            dummy_container = scoretools.Container(self)
            wrapped = True
        mutate(self).swap(new_tuplet)
        if wrapped:
            del(dummy_container[:])
        return new_tuplet

    def _get_crossing_spanners(self):
        r'''Assert logical-voice-contiguous components.
        Collect spanners that attach to any component in selection.
        Returns unordered set of crossing spanners.
        A spanner P crosses a list of logical-voice-contiguous components C
        when P and C share at least one component and when it is the
        case that NOT ALL of the components in P are also in C.
        In other words, there is some intersection -- but not total
        intersection -- between the components of P and C.
        '''
        assert self._all_are_contiguous_components_in_same_logical_voice(self)
        all_components = set(iterate(self).by_class())
        contained_spanners = set()
        for component in iterate(self).by_class():
            contained_spanners.update(component._get_spanners())
        crossing_spanners = set([])
        for spanner in contained_spanners:
            spanner_components = set(spanner[:])
            if not spanner_components.issubset(all_components):
                crossing_spanners.add(spanner)
        return crossing_spanners

    def _get_dominant_spanners(self):
        r'''Returns spanners that dominate components in selection.
        Returns set of (spanner, index) pairs.
        Each (spanner, index) pair gives a spanner which dominates
        all components in selection together with the start index
        at which spanner first encounters selection.
        Use this helper to lift spanners temporarily from components
        in selection and perform some action to the underlying
        score tree before reattaching spanners.
        score components.
        '''
        from abjad.tools import selectiontools
        from abjad.tools import spannertools
        Selection = selectiontools.Selection
        assert self._all_are_contiguous_components_in_same_logical_voice(self)
        receipt = set([])
        if len(self) == 0:
            return receipt
        first, last = self[0], self[-1]
        start_components = first._get_descendants_starting_with()
        stop_components = last._get_descendants_stopping_with()
        stop_components = set(stop_components)
        for component in start_components:
            for spanner in component._get_spanners():
                if set(spanner[:]) & stop_components != set([]):
                    index = spanner.index(component)
                    receipt.add((spanner, index))
        return receipt

    def _get_offset_lists(self):
        start_offsets, stop_offsets = [], []
        for component in self:
            start_offsets.append(component._get_timespan().start_offset)
            stop_offsets.append(component._get_timespan().stop_offset)
        return start_offsets, stop_offsets

    def _give_dominant_spanners(self, recipients):
        r'''Find all spanners dominating music.
        Insert each component in recipients into each dominant spanner.
        Remove music from each dominating spanner.
        Returns none.
        Not composer-safe.
        '''
        assert self._all_are_contiguous_components_in_same_logical_voice(self)
        assert self._all_are_contiguous_components_in_same_logical_voice(
            recipients)
        receipt = self._get_dominant_spanners()
        for spanner, index in receipt:
            for recipient in reversed(recipients):
                spanner._insert(index, recipient)
            for component in self:
                spanner._remove(component)

    def _make_spanner_schema(self):
        schema = {}
        spanners = set()
        for component in iterate(self).by_class():
            spanners.update(component._get_spanners())
        for spanner in spanners:
            schema[spanner] = []
        for i, component in \
            enumerate(iterate(self).by_class()):
            attached_spanners = component._get_spanners()
            for attached_spanner in attached_spanners:
                try:
                    schema[attached_spanner].append(i)
                except KeyError:
                    pass
        return schema

    def _withdraw_from_crossing_spanners(self):
        r'''Not composer-safe.
        '''
        assert self._all_are_contiguous_components_in_same_logical_voice(self)
        crossing_spanners = self._get_crossing_spanners()
        components_including_children = list(iterate(self).by_class())
        for crossing_spanner in list(crossing_spanners):
            spanner_components = crossing_spanner._components[:]
            for component in components_including_children:
                if component in spanner_components:
                    crossing_spanner._components.remove(component)
                    component._spanners.discard(crossing_spanner)

    ### PUBLIC METHODS ###

    def _copy(self, n=1, include_enclosing_containers=False):
        r'''Copies components in selection and fractures crossing spanners.

        Components in selection must be logical-voice-contiguous.

        The steps this function takes are as follows:

            * Deep copy `components`.

            * Deep copy spanners that attach to any component in `components`.

            * Fracture spanners that attach to components not in `components`.

            * Returns Python list of copied components.

        ..  container:: example

            **Example 1.** Copy components one time:

            ::

                >>> staff = Staff(r"c'8 ( d'8 e'8 f'8 )")
                >>> staff.append(r"g'8 a'8 b'8 c''8")
                >>> time_signature = TimeSignature((2, 4))
                >>> attach(time_signature, staff)
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff {
                    \time 2/4
                    c'8 (
                    d'8
                    e'8
                    f'8 )
                    g'8
                    a'8
                    b'8
                    c''8
                }

            ::

                >>> selection = staff.select_leaves()[2:4]
                >>> result = selection._copy()
                >>> new_staff = Staff(result)
                >>> show(new_staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(new_staff)
                \new Staff {
                    e'8 (
                    f'8 )
                }

            ::

                >>> staff.select_leaves()[2] is new_staff.select_leaves()[0]
                False

        ..  container:: example

            **Example 2.** Copy components multiple times:

            Copy `components` a total of `n` times:

            ::

                >>> selection = staff.select_leaves()[2:4]
                >>> result = selection._copy(n=4)
                >>> new_staff = Staff(result)
                >>> show(new_staff) # doctest: +SKIP

            ::

                >>> print format(new_staff)
                \new Staff {
                    e'8 (
                    f'8 )
                    e'8 (
                    f'8 )
                    e'8 (
                    f'8 )
                    e'8 (
                    f'8 )
                }

        ..  container:: example

            **Example 3.** Copy leaves and include enclosing conatiners:

                >>> voice = Voice(r"\times 2/3 { c'4 d'4 e'4 }")
                >>> voice.append(r"\times 2/3 { f'4 e'4 d'4 }")
                >>> staff = Staff([voice])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(staff)
                \new Staff {
                    \new Voice {
                        \times 2/3 {
                            c'4
                            d'4
                            e'4
                        }
                        \times 2/3 {
                            f'4
                            e'4
                            d'4
                        }
                    }
                }

            ::

                >>> leaves = staff.select_leaves(1, 5)
                >>> new_staff = leaves._copy(include_enclosing_containers=True)
                >>> show(new_staff) # doctest: +SKIP

            ..  doctest::

                >>> print format(new_staff)
                \new Staff {
                    \new Voice {
                        \times 2/3 {
                            d'4
                            e'4
                        }
                        \times 2/3 {
                            f'4
                            e'4
                        }
                    }
                }

        Returns contiguous selection.
        '''
        from abjad.tools import scoretools
        from abjad.tools import spannertools
        # check input
        assert self._all_are_contiguous_components_in_same_logical_voice(self)
        # return empty list when nothing to copy
        if n < 1:
            return []
        new_components = [
            component._copy_with_children_and_indicators_but_without_spanners()
            for component in self
            ]
        if include_enclosing_containers:
            return self._copy_and_include_enclosing_containers()
        new_components = type(self)(new_components)
        # make schema of spanners contained by components
        schema = self._make_spanner_schema()
        # copy spanners covered by components
        for covered_spanner, component_indices in schema.items():
            new_covered_spanner = copy.copy(covered_spanner)
            del(schema[covered_spanner])
            schema[new_covered_spanner] = component_indices
        # reverse schema
        reversed_schema = {}
        for new_covered_spanner, component_indices in schema.items():
            for component_index in component_indices:
                try:
                    reversed_schema[component_index].append(
                        new_covered_spanner)
                except KeyError:
                    reversed_schema[component_index] = [new_covered_spanner]
        # iterate components and add new components to new spanners
        for component_index, new_component in enumerate(
            iterate(new_components).by_class()):
            try:
                new_covered_spanners = reversed_schema[component_index]
                for new_covered_spanner in new_covered_spanners:
                    new_covered_spanner.append(new_component)
            except KeyError:
                pass
        # repeat as specified by input
        for i in range(n - 1):
            new_components += self._copy()
        # return new components
        return new_components

    def get_timespan(self, in_seconds=False):
        r'''Gets timespan of contiguous selection.

        Returns timespan.
        '''
        from abjad.tools import timespantools
        if in_seconds:
            raise NotImplementedError
        start_offset = min(x._get_timespan().start_offset for x in self)
        stop_offset = max(x._get_timespan().stop_offset for x in self)
        return timespantools.Timespan(start_offset, stop_offset)

    def group_by(self, predicate):
        '''Groups components in contiguous selection by `predicate`.

        Returns list of tuples.
        '''
        result = []
        grouper = itertools.groupby(self, predicate)
        for label, generator in grouper:
            selection = tuple(generator)
            result.append(selection)
        return result

    def partition_by_durations(
        self,
        durations,
        cyclic=False,
        fill='exact',
        in_seconds=False,
        overhang=False,
        ):
        r'''Partitions `components` according to `durations`.

        When `fill` is ``'exact'`` then parts must equal `durations` exactly.

        When `fill` is ``'less'`` then parts must be
        less than or equal to `durations`.

        When `fill` is ``'greater'`` then parts must be
        greater or equal to `durations`.

        Reads `durations` cyclically when `cyclic` is true.

        Reads component durations in seconds when `in_seconds` is true.

        Returns remaining components at end in final part when `overhang`
        is true.
        '''
        # TODO: decide on correct assertion
        #assert self._all_are_contiguous_components_in_same_logical_voice(self)
        durations = [durationtools.Duration(x) for x in durations]
        if cyclic:
            durations = datastructuretools.CyclicTuple(durations)
        result = []
        part = []
        current_duration_index = 0
        target_duration = durations[current_duration_index]
        cumulative_duration = durationtools.Duration(0)
        components_copy = list(self)
        while True:
            try:
                component = components_copy.pop(0)
            except IndexError:
                break
            component_duration = component._get_duration()
            if in_seconds:
                component_duration = component._get_duration(in_seconds=True)
            candidate_duration = cumulative_duration + component_duration
            if candidate_duration < target_duration:
                part.append(component)
                cumulative_duration = candidate_duration
            elif candidate_duration == target_duration:
                part.append(component)
                result.append(part)
                part = []
                cumulative_duration = durationtools.Duration(0)
                current_duration_index += 1
                try:
                    target_duration = durations[current_duration_index]
                except IndexError:
                    break
            elif target_duration < candidate_duration:
                if fill == 'exact':
                    raise PartitionError
                elif fill == 'less':
                    result.append(part)
                    part = [component]
                    if in_seconds:
                        cumulative_duration = \
                            sum([x._get_duration(in_seconds=True)
                            for x in part])
                    else:
                        cumulative_duration = \
                            sum([x._get_duration() for x in part])
                    current_duration_index += 1
                    try:
                        target_duration = durations[current_duration_index]
                    except IndexError:
                        break
                    if target_duration < cumulative_duration:
                        message = 'target duration {}'
                        message += ' is less than cumulative duration {}.'
                        message = message.format(
                            target_duration, cumulative_duration)
                        raise PartitionError(message)
                elif fill == 'greater':
                    part.append(component)
                    result.append(part)
                    part = []
                    cumulative_duration = durationtools.Duration(0)
                    current_duration_index += 1
                    try:
                        target_duration = durations[current_duration_index]
                    except IndexError:
                        break
        if len(part):
            if overhang:
                result.append(part)
        if len(components_copy):
            if overhang:
                result.append(components_copy)
        return result

    def partition_by_durations_exactly(
        self,
        durations,
        cyclic=False,
        in_seconds=False,
        overhang=False,
        ):
        r'''Partitions components in selection by `durations` exactly.


        Returns list of selections.
        '''
        return self.partition_by_durations(
            durations,
            cyclic=cyclic,
            fill='exact',
            in_seconds=in_seconds,
            overhang=overhang,
            )

    def partition_by_durations_not_greater_than(
        self,
        durations,
        cyclic=False,
        in_seconds=False,
        overhang=False,
        ):
        r'''Partitions components in selection by values of durations not
        greater than those in `durations`.

        Returns list of selections.
        '''
        return self.partition_by_durations(
            durations,
            cyclic=cyclic,
            fill='less',
            in_seconds=in_seconds,
            overhang=overhang,
            )

    def partition_by_durations_not_less_than(
        self,
        durations,
        cyclic=False,
        in_seconds=False,
        overhang=False,
        ):
        r'''Partitions components in selection by values of durations not less
        than those in `durations`.

        Returns list of selections.
        '''
        return self.partition_by_durations(
            durations,
            cyclic=cyclic,
            fill='greater',
            in_seconds=in_seconds,
            overhang=overhang,
            )



# -*- encoding: utf-8 -*-
from abjad import *


def test_pitchtools_OctaveTranspositionMapping_name_01():

    mapping = pitchtools.OctaveTranspositionMapping()
    assert mapping.custom_identifier is None

    mapping.custom_identifier = 'lower register mapping #2'
    assert mapping.custom_identifier == 'lower register mapping #2'



# -*- encoding: utf-8 -*-
from abjad import *


def test_spannertools_Hairpin_start_dynamic_string_01():

    staff = Staff("c'8 d'8 e'8 f'8")
    hairpin = Hairpin(descriptor='p < f')
    attach(hairpin, staff[:])

    r'''
    \new Staff {
        c'8 \< \p
        d'8
        e'8
        f'8 \f
    }
    '''

    assert hairpin.start_dynamic_string == 'p'
    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            c'8 \< \p
            d'8
            e'8
            f'8 \f
        }
        '''
        )

    hairpin.start_dynamic_string = 'mf'

    r'''
    \new Staff {
        c'8 \< \mf
        d'8
        e'8
        f'8 \f
    }
    '''

    assert hairpin.start_dynamic_string == 'mf'
    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            c'8 \< \mf
            d'8
            e'8
            f'8 \f
        }
        '''
        )



# -*- encoding: utf-8 -*-
'''Abjad ``scoretools`` package.

    Dependencies:

    The ``scoretools`` package may import the ``scoretools``
    and ``componenttool`` packages at top level.

    The ``scoretools`` package should not import sibling
    or cousing packages like ``scoretools`` at top level.

    The ``scoretools`` package should not import ``instrumenttools`` 
    at top level.
    
'''
from abjad.tools import systemtools

systemtools.ImportManager.import_structured_package(
	__path__[0],
	globals(),
	)

_documentation_section = 'core'



# -*- encoding: utf-8 -*-
import pytest
from abjad import *


def test_pitchtools_transpose_named_pitch_by_numbered_interval_and_respell_01():

    pitch = NamedPitch(0)

    assert pitchtools.transpose_named_pitch_by_numbered_interval_and_respell(pitch, 1, 0) == NamedPitch('dff', 4)
    assert pitchtools.transpose_named_pitch_by_numbered_interval_and_respell(pitch, 1, 0.5) == NamedPitch('dtqf', 4)
    assert pitchtools.transpose_named_pitch_by_numbered_interval_and_respell(pitch, 1, 1) == NamedPitch('df', 4)
    assert pitchtools.transpose_named_pitch_by_numbered_interval_and_respell(pitch, 1, 1.5) == NamedPitch('dqf', 4)
    assert pitchtools.transpose_named_pitch_by_numbered_interval_and_respell(pitch, 1, 2) == NamedPitch('d', 4)
    assert pitchtools.transpose_named_pitch_by_numbered_interval_and_respell(pitch, 1, 2.5) == NamedPitch('dqs', 4)
    assert pitchtools.transpose_named_pitch_by_numbered_interval_and_respell(pitch, 1, 3) == NamedPitch('ds', 4)
    assert pitchtools.transpose_named_pitch_by_numbered_interval_and_respell(pitch, 1, 3.5) == NamedPitch('dtqs', 4)
    assert pitchtools.transpose_named_pitch_by_numbered_interval_and_respell(pitch, 1, 4) == NamedPitch('dss', 4)

    statement = 'pitchtools.transpose_named_pitch_by_numbered_interval_and_respell(pitch, 1, 4.5)'
    assert pytest.raises(KeyError, statement)


def test_pitchtools_transpose_named_pitch_by_numbered_interval_and_respell_02():

    pitch = NamedPitch(0)

    assert pitchtools.transpose_named_pitch_by_numbered_interval_and_respell(pitch, -1, 0) == NamedPitch('bs', 3)
    assert pitchtools.transpose_named_pitch_by_numbered_interval_and_respell(pitch, -1, -0.5) == NamedPitch('bqs', 3)
    assert pitchtools.transpose_named_pitch_by_numbered_interval_and_respell(pitch, -1, -1) == NamedPitch('b', 3)
    assert pitchtools.transpose_named_pitch_by_numbered_interval_and_respell(pitch, -1, -1.5) == NamedPitch('bqf', 3)
    assert pitchtools.transpose_named_pitch_by_numbered_interval_and_respell(pitch, -1, -2) == NamedPitch('bf', 3)
    assert pitchtools.transpose_named_pitch_by_numbered_interval_and_respell(pitch, -1, -2.5) == NamedPitch('btqf', 3)
    assert pitchtools.transpose_named_pitch_by_numbered_interval_and_respell(pitch, -1, -3) == NamedPitch('bff', 3)
